# -*- coding: utf-8 -*-
from __future__ import unicode_literals

#Les variables suivantes sont déclarées dans __init__.py :
# connexion = None
# databasedocument = None
# colonnesPos = []


import os
from distutils import dir_util, file_util

from com.sun.star.awt import KeyEvent
from com.sun.star.awt.KeyModifier import SHIFT, MOD1, MOD2
from com.sun.star.document.MacroExecMode import ALWAYS_EXECUTE_NO_WARN

from libtooournoi import utils


O = utils.Office()
cles = O.getRegistryKeyContent("org.openoffice.tooournoi.Config", True)

def demarrerTOOournoi():
    # return
    if cles.FirstRun:
        setFirstRun(False)
        if O.msgbox('Vous installez "tOOournoi" pour la première fois.\n\n'
                    'Un sous-répertoire <%s> sera créé dans le répertoire\t\n'
                    '-->  %s  <--\n\n'
                    'Pour accepter ce chemin par défaut, cliquez sur OUI.\n'
                    'Pour modifier ce chemin par défaut, cliquez sur NON.\n\n' %
                    (cles.SubPath, O.convertFromUrl(O.substitutePath(cles.Path))),
                    'Installation de tOOournoi', boutons=('yesno',)) == 3:
            optionsTOOournoi(False)
        modifierSecureUrl("", getTOOournoiPath())

    autoriserExtensionReferences()
    extPath = O.convertFromUrl(getExtPath())
    tooournoipath = O.convertFromUrl(getTOOournoiPath())
    version = getExtVer()

    fichiers = ("tOOournoi.odb", "Manuel.pdf", "notes de versions.txt", "lisez-moi.txt")
    fichiersver = ("tOOournoi_%s.odb" % version, "Manuel_%s.pdf" % version, "notes de versions_%s.txt" % version, "lisez-moi.txt")
    if not os.path.isdir(tooournoipath): os.makedirs(tooournoipath)
    for n in range(len(fichiers)):
        file_util.copy_file(os.path.join(extPath, fichiers[n]), os.path.join(tooournoipath, fichiersver[n]), update=True)
    if not os.path.isdir(os.path.join(tooournoipath, "database2")):
        dir_util.copy_tree(os.path.join(extPath, "database"), os.path.join(tooournoipath, "database2"))
    O.loadComponentFromURL(getTOOournoiPath() + fichiersver[0])

def demarrageApplication(event):
    '''Ouvre le menu principal au lancement de l'application'''
    # mettre à jour le contexte
    global O
    O.frame = O.getDesktop().getCurrentFrame()
    NOM = "tOOournoi"
    doc = event.Source
    url = doc.URL
    oBaseContext = O.createUnoService("com.sun.star.sdb.DatabaseContext")
    bdp = O.barreDeProgression('Démarrage...\t', 100)
    bdp.start()
    bdp.setValue(25)
    # pour éviter certains problèmes chiants (notamment fichier source effacé entre temps)
    # on enregistre la base à chaque ouverture (c'est bourrin mais rapide)
    if oBaseContext.hasByName(NOM):
        oBaseContext.revokeDatabaseLocation(NOM)
    oBaseContext.registerDatabaseLocation(NOM, url)
    dataSource = oBaseContext.getByName(NOM)
    try:
        # ' fixe la taille de la fenêtre
        coord = (190,150,880,520,15)
        doc.CurrentController.ApplicationMainWindow.setPosSize( *coord )
        bdp.setValue(50)
        # ' établit la connexion et ouvre le menu principal
        dbPath = ("jdbc:hsqldb:file:%s"
                  ";default_schema=true;shutdown=true;hsqldb.default_table_type=cached;get_column_name=false" %
                  O.convertFromUrl("%sdatabase2/tooournoi"% getTOOournoiPath()))
        driverClass = "org.hsqldb.jdbcDriver"
        driverClassPath = O.substitutePath("$(prog)/classes/hsqldb.jar", False)
        props = O.createPropertyValues(JavaDriverClass=driverClass,
                                       JavaDriverClassPath=driverClassPath)
        dataSource.URL = dbPath
        dataSource.Info = props
        doc.CurrentController.connect()
        utils.databasedocument = doc
        utils.connexion = doc.CurrentController.ActiveConnection
        bdp.setValue(75)
        # ' besoin de la classe Tournoi à partir d'ici (ne pas l'instancier avant la connexion !)
        T = utils.Tournoi()
        T.modifierNombreParties(T.infos['maxpartie'])
        menuprincipal = T.ouvreForm("Menu_Principal")
        # ' affichage paires si nécessaire
        tc = menuprincipal.DrawPage.Forms.getByName("Standard_Menu_Inscriptions").getByName("TableControl")
        tc.ColClub.Hidden = T.infos['paires']
        tc.ColInscrits.Hidden = T.infos['paires']
        tc.ColJoueur1.Hidden = not T.infos['paires']
        tc.ColJoueur2.Hidden = not T.infos['paires']
        # ' définit les raccourcis clavier pour navigation onglet - routine ci-dessous
        # ' (en attendant de pouvoir les paramétrer de manière définitive)
        creeShortcuts(menuprincipal)
        bdp.setValue(100)
    finally:
        bdp.end()

def setFirstRun(bool=True):
    '''Modifie la clef de registre 'FirstRun'.'''
    cles.FirstRun = bool
    cles.commitChanges()
    cles.dispose()

def optionsTOOournoi(bMoveRep=True, bNoRep=False):
    '''Modifie les clefs de registre 'Path' et 'SubPath'.
    bMoveRep : pour déplacer le répertoire de tOOournoi
    bNoRep : pour ne pas donner accès au modif de répertoire'''
    setFirstRun(False)
    CurRep = getTOOournoiPath()
    modifierSecureUrl("", CurRep)
    subPath = cles.SubPath
    # création du dialogue
    dlg = O.createUnoDialog("vnd.sun.star.script:tOOournoi.dialogue7?location=application")
    txtPath = dlg.getControl("txtPath")
    txtSubPath = dlg.getControl("txtSubPath")
    txtSubPathEditable = dlg.getControl("txtSubPathEditable")
    txtSendMailTo = dlg.getControl("txtSendMailTo")
    txtPlayersUpdateUrl = dlg.getControl("txtPlayersUpdateUrl")
    groupPath = dlg.getControl("groupPath")
    txtPath.setText( O.convertFromUrl(O.substitutePath(cles.Path)) )
    txtSubPath.setText(subPath)
    txtSubPathEditable.setText(subPath)
    txtSendMailTo.setText(cles.SendMailTo)
    txtPlayersUpdateUrl.setText( cles.PlayersUpdateUrl )
    groupPath.Model.Label = "Chemin du sous-répertoire <%s>" % subPath
    if bNoRep:
        txtPath.Enable = False
        txtSubPath.Enable = False
        dlg.getControl("boutonPath").Enable = False
        dlg.getControl("boutonSubPath").Enable = False
    if dlg.execute():   # si changements validés
        # modif du registre
        cles.setPropertyValue("SendMailTo", txtSendMailTo.getText())
        cles.setPropertyValue("PlayersUpdateUrl", txtPlayersUpdateUrl.getText())
        cles.commitChanges()
        NouvRep = getTOOournoiPath()
        if NouvRep != CurRep:   # si le nouveau répertoire diffère de l'ancien
            modifierSecureUrl(CurRep, NouvRep)
            if bMoveRep:    #' si le déplacement du sous-répertoire est demandé
                if os.path.isdir(O.convertFromUrl(CurRep)):   #' et si un ancien répertoire existe
                    # ' il faut préalablement désenregistrer la base...
                    oBaseContext = O.createUnoService("com.sun.star.sdb.DatabaseContext")
                    if oBaseContext.hasByName("tOOournoi"): oBaseContext.revokeObject("tOOournoi")
                    # '...avant de déplacer puis supprimer la source
                    dir_util.copy_tree(O.convertFromUrl(CurRep), O.convertFromUrl(NouvRep))
                    dir_util.remove_tree(O.convertFromUrl(CurRep))

def modifRepTOOournoi():
    '''Routine d'accès aux options de tOOournoi'''
    baseURL = '%stOOournoi_%s.odb' % (getTOOournoiPath(), getExtVer())
    if O.docSiOuvert(baseURL):
        optionsTOOournoi(False, True)
    else:
        optionsTOOournoi()

def modifTxtPath(event):
    '''Enregistre le répertoire de travail de tOOournoi'''
    desc = "\nModifier le répertoire d'installation de tOOournoi :\n"
    oFP = O.createUnoService("com.sun.star.ui.dialogs.FolderPicker")
    txtPath = event.Source.Context.getControl('txtPath')
    # on error resume next
    oFP.setDisplayDirectory(O.convertToUrl(txtPath.Text))
    oFP.setDescription(desc)
    if oFP.execute():
        while 1:
            nouvPath = O.convertFromUrl(oFP.getDirectory())
            if os.path.isdir(nouvPath):    # si le choix est bien un répertoire
                txtPath.setText(nouvPath)
                cles.setPropertyValue("Path", O.substitutePath(nouvPath, True))   # prépare la modif du registre
                break
            else:
                O.msgbox("Ce répertoire n'est pas un répertoire valide.", "Erreur", 'warn')

def modifTxtSubPath(event):
    '''Le sous-répertoire dédié à tOOournoi'''
    source = event.Source
    dlg = source.Context
    groupPath = dlg.getControl('groupPath')
    txtSubPath = dlg.getControl('txtSubPath')
    txtSubPathEditable = dlg.getControl('txtSubPathEditable')
    if "Retour" in source.Model.Name:   # bouton 'retour'
        nouvSubPath = ''.join(x if x.isalnum() or x in ' -()[]' else '_' for x in txtSubPathEditable.Text.strip())
        if nouvSubPath:
            txtSubPath.setText(nouvSubPath)
            txtSubPathEditable.setText(nouvSubPath)
            groupPath.Model.Label = "Chemin du sous-répertoire <%s>" % nouvSubPath
            cles.setPropertyValue("SubPath", nouvSubPath)   # prépare la modif du registre
        dlg.Model.Step = 1
    else:
        dlg.Model.Step = 2
        # ' donne le focus et sélectionne le champ
        txtSubPathEditable.setFocus()
        sel = txtSubPathEditable.getSelection()
        sel.Max = len(subPath)
        txtSubPathEditable.setSelection(sel, True)

def modifierSecureUrl(oldRep, newRep):
    '''Ajoute le répertoire 'tOOournoi' aux répertoires de confiance
    (Outils -> Options -> OpenOffice.org -> Sécurité)'''
    RKC = O.getRegistryKeyContent("org.openoffice.Office.Common/Security/Scripting", True)
    secureURL = list(RKC.SecureURL) if RKC.SecureURL else []
    if oldRep: secureURL.remove(oldRep)
    if newRep: secureURL.append(newRep)
    RKC.SecureURL = tuple(set(secureURL))
    RKC.commitChanges()
    
def autoriserExtensionReferences():
    '''Garantit que l'option "Étendre les références lors de l'insertion de colonnes/lignes
    soit toujours TRUE (requis pour l'affichage correct des divers classeurs)'''
    RKC = O.getRegistryKeyContent("org.openoffice.Office.Calc/Input", True)
    RKC.ExpandReference = True
    RKC.commitChanges()
    
def getTOOournoiPath():
    '''Renvoie le répertoire de travail de tOOournoi'''
    sPath = cles.Path
    if sPath[-1] != "/": sPath += "/"
    return O.substitutePath(sPath + cles.SubPath + "/")

def getExtPath():
    '''Renvoie le répertoire d'installation de tOOournoi'''
    pip = O.ctx.getByName("/singletons/com.sun.star.deployment.PackageInformationProvider")
    extPath = pip.getPackageLocation(O.extID)
    if extPath[-1] != "/": extPath += "/"
    return extPath

def getExtVer():
    '''Fonction renvoyant la version de tOOournoi.'''
    pip = O.ctx.getByName("/singletons/com.sun.star.deployment.PackageInformationProvider")
    extensions = pip.getExtensionList()
    for e in extensions:
        if O.extID in e:
            return e[1]
    return ''

def creeShortcuts(doc):
    from com.sun.star.awt.Key import A, I, P, O
    KE = KeyEvent()
    KE.Modifiers = MOD2
    SKE = doc.UIConfigurationManager.ShortCutManager
    onglets = {"A":A, "I":I, "P":P, "O":O}
    script = "vnd.sun.star.script:tOOournoi.oxt|python|main.py$changeOnglet_Alt_%s?language=Python&location=user:uno_packages"
    for k in onglets:
        KE.KeyCode = onglets[k]
        SKE.setKeyEvent(KE, script % k)

def fermetureMenuPrincipal(event):
    '''Sauvegarde et ferme l'application'''
    T = utils.Tournoi()
    if T.msgbox("\nVous êtes sur le point de quitter le programme.\t\n\n\n"
                "Souhaitez-vous poursuivre ?\n\t\n\n",
                "Confirmation", 'warn', boutons=('yesno','defno')) == 2:
        for form in T.iterUno(utils.databasedocument.FormDocuments):
            if form.Component: form.dispose()

def fermetureApplication(event):
    T = utils.Tournoi()
    # return 
    try:
        forms = T.doc.FormDocuments
    except AttributeError:
        pass
    else:
        if forms.getByName("Menu_Principal").Component: return
        T.cree_archives()
        if T.doc.isModified(): T.doc.store() 
        # T.connexions.clear()
        # T.doc.CurrentController.ActiveConnection.close()
        # T.doc.DataSource.dispose()
        if len(list(T.iterUno(T.getDesktop().Components))) > 1:
            T.executeDispatch(".uno:CloseDoc")
        else:
            T.executeDispatch(".uno:Quit")

# lists the scripts, that shall be visible inside OOo. Can be omited, if
# all functions shall be visible
g_exportedScripts = (demarrerTOOournoi, modifTxtPath, modifTxtSubPath,
                     demarrageApplication, fermetureMenuPrincipal, fermetureApplication)
