#!/usr/bin/python
# -*- coding: utf-8 -*-

LANG = "latin-1"
LG_MAX=15
MAX_MOT=21
NB_COUP=36
NB_JOUEUR=100
NB_LETTRE=27
NB_TIR=10
NB_TOP=500
DISTRIBUTION = ('?','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z')
LIGNE = ('A','B','C','D','E','F','G','H','I','J','K','L','M','N','O')
PARTICUDIC = {0:'', 1: 'a', 2:'?',10:'s', 30: 'z',100:'abs'}

def particudic(x):
  try:
    return PARTICUDIC[x] 
  except KeyError:
    print "key inconnue dans PARTICUDIC: %d" % x
    return '?'

def negatize(pts,top):
  """
  Retourne les pts d'un coup sous forme de négatif (str)
  """
  if pts==top:
    return 'Top'
  else:
    return "%+d" % (pts - top)

class Joueur(object):
  """
  Objet qui représente un joueur d'une partie
  Taille =  677 bytes
  """
  size = 677
  def __init__(self,s=None):
    self.originalstring = s
    if s:
      self._parse()
    
  def _parse(self):  	
    from struct import unpack
    s = self.originalstring[:61] 
    self.nom = s[:s.find('\x00')].decode('latin-1') # nom prénom
    s=self.originalstring[61:69]
    if not self.tablevide():
      self.licence =  int(s[:s.find('\x00')]) # licence
    else:
      self.licence = 0
    s=self.originalstring[69:72]
    self.serie =  s[:s.find('\x00')] # série
    s=self.originalstring[72:78]
    self.club =  s[:s.find('\x00')] # club
    s=self.originalstring[78:80]
    self.categorie =  s[:s.find('\x00')] # catégorie
    s=self.originalstring[80:83]
    self.fede =  s[:s.find('\x00')] # fede
    s=self.originalstring[83:88]
    self.table =  s[:s.find('\x00')] # table
    self.tablefixe = (unpack('b',self.originalstring[88])[0] == 1)# vrai si table fixe
    self.ptscoups = unpack('i'*37,self.originalstring[89:237]) #pts de chaque coups
    self.ptssolo = [x==1 for x in unpack('b'*37,self.originalstring[237:274])]# vrai si des pts de solo ont été attribués par coup
    self.ptsmodif = [x==1 for x in unpack('b'*37,self.originalstring[274:311])]# vrai si le score du coup a été modifié     
    self.particularites = unpack('B'*37,self.originalstring[311:348]) #particularités du coup
    self.cumuls = unpack('i'*37,self.originalstring[348:496]) #cumul coup par coup
    self.nbavertissements = unpack('b', self.originalstring[496])[0] #nb d'avertissements reçus
    self.nbpenalites = unpack('b', self.originalstring[497])[0] # nb des pénalités reçues
    self.ptsrevendiques = unpack('i', self.originalstring[498:502])[0] # pts revendiqués ??????????????
    self.litige = unpack('b',self.originalstring[502])[0] # vrai si joueur en litige ?????????????
    self.openbrol = unpack('b',self.originalstring[503])[0] # brol
    self.ptsavant = unpack('i', self.originalstring[504:508])[0] # pts acquis précédemment
    #on laisse tomber 86 bytes car brol trap => 594
    # coordonnées du partenaire en paires
    s = self.originalstring[594:655] 
    self.nom2 = s[:s.find('\x00')] # nom prénom
    s=self.originalstring[655:663]
    self.licence2 =  s[:s.find('\x00')] # licence
    s=self.originalstring[663:666]
    self.serie2 =  s[:s.find('\x00')] # série
    s=self.originalstring[666:672]
    self.club2 =  s[:s.find('\x00')] # club
    s=self.originalstring[672:674]
    self.categorie2 =  s[:s.find('\x00')] # catégorie
    s=self.originalstring[674:677]
    self.fede2 =  s[:s.find('\x00')] # fede
    
  def __repr__(self):
    sz = "%s - %s - %s - %s - %s - %s\n" % (self.table,self.nom,self.club, self.serie,self.categorie,self.fede)
    sz += "%d : "  % self.cumuls[-2]+ " - ".join([str(c) for c in self.ptscoups if c > -1])
    return sz
  def get_nom(self):
    return self.nom
  
  def get_club(self):
    return self.club
  
  def get_serie(self):
    """
    Retourne le n° de la série
    """
    if self.serie[0] in ('1','2','3','4','5','6','7'):
      return self.serie[0]
    else:
      return '-'
  
  def get_fullserie(self):
    """
    Retourne la série complète (n° + lettre éventuelle)
    """
    if self.serie[0] in ('1','2','3','4','5','6','7'):
      return self.serie
    else:
      return '-'
  
  def get_categorie(self):
    if self.categorie in ('P','B','C','J','E','V','D','-'):
      return self.categorie
    else:
      return 'S'
  
  def get_table(self):
    return self.table
  
  def get_cumul(self,cp):
    """
    Retourne le cumul au coup cp
    """
    return self.cumuls[cp-1]

  def get_neg(self,cp,top):
    """
    Retourne le négatif
    """
    return top-self.cumuls[cp-1]

  def show(self):
    print "%s - %d - %s - %s" % (self.nom,self.licence,self.serie,self.club)
    sztitre = "cp  |"
    szpts = "pts |"
    szcumul = "cum |"
    for i in range(1,len(self.ptscoups) + 1):
      sztitre += "%s|" % (str(i).center(5))
    idx=0
    for c in zip(self.ptscoups,self.cumuls):
      if c[0] == -1:
        break
      szpts += "%s|" % (str(c[0]).center(5))
      szcumul += "%s|" % (str(c[1]).center(5))
    print sztitre
    print szpts
    print szcumul
      
  def joueur(self):
    """
    retourne un tuple (table,nom,licence,serie,club)
    """
    return self.table,self.nom,self.licence,self.serie,self.club
  
  def joueurcoups(self,cp):
    """
    Retourne un tuple (table,nom,[coups])
    """
    return self.table,self.nom,[x for x in self.ptscoups[:cp]]
  
  def tablevide(self):
    """
    Retourne un booléen indiquant si la table est vide ou pas
    """
    return (self.nom == '' or self.nom == '--table vide--')
      
  def joucumul(self,cp):
    """
    Retourne un tuple (table,nom,cumul)
    """
    return self.table,self.nom,self.cumuls[cp-1]

  def joucumulneg(self,cp,top):
    """
    Retourne un tuple (table,nom,cumul)
    """
    return self.table,self.nom,negatize(self.cumuls[cp-1],top)

  def tablecumul(self,cp):
    """
    Retourne un tuple (table,cumul)
    """
    return self.table,self.cumuls[cp-1]

  def fdr(self,cp):
    """
    retourne une liste de tuples[(pts,av,cumul),...]
    """
    ptcps = [x for x in self.ptscoups[:cp]]
    av = [x for x in self.particularites[:cp]]
    cumuls = [x for x in self.cumuls[:cp]]
    return zip(ptcps,av,cumuls)
      
  def fdrtable(self,cp):
    """
    retourne un tuple ("numtable",top,[(pts,av),...])
    av = 1 si av
    av = 2 si pénalité
    av = 10 si solo
    av = 11 si av et solo
    av = 12 si av et pénalité
    av = 30 si zéro
    """
    ptcps = [x for x in self.ptscoups[:cp]]
    av = [x for x in self.particularites[:cp]]
    cps = zip(ptcps,av)
    return str(self.table),self.cumuls[cp-1],cps
  
  def tablecoups(self,cp):
    """
    Retourne une liste de tuples des tables +  liste des tuples pts,avert (a ou p)
    (table,[[pts,'av'],..])
    """
    ptcps = [x for x in self.ptscoups[:cp]]
    av = [particudic(x) for x in self.particularites[:cp]]
    return self.table,zip(ptcps,av)
      
  def tablecoupspts(self,cp):
    """
    Retourne une liste de tuples des tables + tablefixe + liste des pts
    (table,tablefixe,[pts,..])
    """
    return self.table,self.tablefixe,[x for x in self.ptscoups[:cp]]

  def ptscoup(self,cp):
    """
    Retourne les pts obtenus au coup cp
    """
    return self.ptscoups[cp-1]
    
class Coup(object):
  """
  Objet qui représente un coup d'une partie
  Taille = 75 bytes
  """
  size = 75
  def __init__(self,s=None,cp=0):
    self.num = cp
    self.originalstring = s
    if s:
      self._parse()
    
  def _parse(self):
    from struct import unpack
    self.sens = unpack('b',self.originalstring[0])[0] # 0=H, 1=V
    self.ligne = LIGNE[unpack('b',self.originalstring[1])[0]] # commence à 0
    self.colonne = unpack('b',self.originalstring[2])[0] + 1 # commence à 0
    s = self.originalstring[3:19]
    self.mot = s[:s.find('\x00')]# mot joué
    self.compte = unpack('i',self.originalstring[19:23])[0] #pts du mot
    self.PosJoker = unpack('cccc',self.originalstring[23:27]) #Position du/des jokers (Champ binaire)
    self.NewMot = (unpack('b',self.originalstring[27])[0] == 1) #indique si le mot est nouveau
    self.scrabble = (unpack('b',self.originalstring[28])[0] == 1) #indique si c'est un scrabble
    self.rang = unpack('i',self.originalstring[29:33])[0]#Rang du coup joué dans TabTop
    s = self.originalstring[33:44] 
    self.tirage = s[:s.find('\x00')] # tirage du coup
    s = self.originalstring[44:55] 
    self.reliquat = s[:s.find('\x00')] # reliquat du coup
    self.nbreliquat = unpack('b',self.originalstring[55])[0] # Nb de lettres du tirage précédent
    self.rejet = (unpack('b',self.originalstring[56])[0] == 1)
    self.CumulPartie = unpack('i',self.originalstring[57:61])[0] # cumul de la partie
    self.Defok = (unpack('b',self.originalstring[61])[0] == 1) # Définition méritant d'être imprimée
    self.TirageParfait = (unpack('b',self.originalstring[62])[0] == 1) # Vrai si tirage validé
    self.DejaCherche = (unpack('b',self.originalstring[63])[0] == 1) #bool DejaCherche; // Vrai si déjà cherché
    self.DejaJoue = (unpack('b',self.originalstring[64])[0] == 1)  #bool DejaJoue; // Vrai si le mot a été posé et comptabilisé
    self.ArbitrageFait = (unpack('b',self.originalstring[65])[0] == 1) #bool ArbitrageFait; // Vrai si toutes les tables sont saisies
    self.Force = (unpack('b',self.originalstring[66])[0] == 1) #bool Force; // Coup forcé en partie jeune
    self.SStop = unpack('i',self.originalstring[67:71])[0] #pts du sstop si renseigné, -1 sinon
    self.moyenne = unpack('i',self.originalstring[71:])[0] #pts de la moyenne    
    
  def __repr__(self):
    if self.reliquat =="":
      sz = self.tirage + " => "
    else:
      if self.rejet:
        sz = "-" + self.tirage + " => "
      else:
        sz = "(%s) %s => " % (self.reliquat,self.tirage)
    if self.scrabble:
      sz += "$" 
    else:
      sz += " "
    if self.NewMot:
      sz += "*"
    else:
      sz += " "
    sz += self.mot
    if self.sens == 0:
      sz += " - " + self.ligne + str(self.colonne)
    else:
      sz += " - " + str(self.colonne) + self.ligne
    sz += " - %d => %d" %(self.compte,self.CumulPartie)
    return sz
  def alphanum(self):
    if self.sens==1:
      return "%d%c" % (self.colonne,self.ligne)
    else:
      return "%c%d" % (self.ligne,self.colonne)

  def show(self):
    print "%d: %s - %s - %d - %d" % (self.num,self.mot,self.alphanum(),self.compte,self.CumulPartie)
  
  def partie(self):
    """
    Retourne un tuple (num du coup,tirage,solution,référence,pts,cumul)
    """
    return self.num,self.tirage,self.mot,self.alphanum(),self.compte,self.CumulPartie
  
class Duplitop(object):
  def __init__(self,nom,f=None):
    self.nom = nom
    self.parsed = 0 #utilisé dans duplilive_ic
    self.setfile(f)
      
  def setfile(self,f):
    """
    set le fichier Dupltiop
    >>> d=Duplitop('test','exemple.HOP')
    """
    if f:
      self.fichier = f
      self.coups = []
      self.joueursdic = {} # dic des paires {numtable: joueur}

  def open(self):
    """
    Ouvre le fichier Duplitop en lecture binaire
    
    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    """
    try:
      if self.fichier != '':
        self.openfile=open(self.fichier,'rb')
        return True
      return False
    except:
      return False
      
  def ready(self):
    """
    Retourne l'état de l'objet Duplitop càd True si le fichier est créé avec quelquechose d'écrit dedans
    Le deuxième booléen indique s'il y a eu une exception lors du test et donc un problème de communication
    
    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.ready()
    (True, True)
    """
    from os import stat
    try:
      return (stat(self.fichier).st_size > 0),True
    except:
      return False,False
  

  def read(self):
    """
    Lit le fichier duplitop ouvert
    
    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True    
    """
    try:
      self.openfile.seek(0)
      self.fread = self.openfile.read()
      return True
    except:
      return False
      
  
  def parsefile(self):
    """
    Parse le fichier duplitop
    
    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True    
    >>> d.parsefile()
   True   
    """
    from struct import unpack
    self.logiciel=self.fread[:8] #DUPLITOP
    self.version = unpack('I',self.fread[8:12]) [0]# version: 5
    self.versionsafe = unpack('L',self.fread[12:16]) [0]# version sauvegarde > 2
    # on laisse tomber les parametres new_partie [70491:70510]
    s = self.fread[70510:70571]
    self.zearbitre = s[:s.find('\x00')] # nom prénom de l'arbitre
    self.inscrits1 = unpack('L'*100,self.fread[70571:70971]) #licence des inscrits
    self.inscrits2 = unpack('L'*100,self.fread[70971:71371]) #licence des inscrits en paires
    self.premieretable = unpack('i',self.fread[71371:71375])[0] # valeur de la première table arbitrée
    self.nbtables = unpack('i',self.fread[71375:71379])[0]# nb de tables arbitrées
    self.nbinscriptions = unpack('i',self.fread[71379:71383])[0] # nb d'inscriptions
    self.nbfixes = unpack('b',self.fread[71383])[0] # nb de tables fixes
    self.nbvides = unpack('b',self.fread[71384])[0] # nb de tables vides
    self.m = unpack('i',self.fread[71385:71389])[0] # M du tournoi
    self.nbclass4 = unpack('i',self.fread[71389:71393])[0] # nb jou série >= 4
    # param ic
    s = self.fread[71393:71399]
    self.club1 = s[:s.find('\x00')] # club 1
    s = self.fread[71399:71405]
    self.club2 = s[:s.find('\x00')] # club 2
    s = self.fread[71405:71430]
    self.equipe1 = s[:s.find('\x00')] # nom équipe 1
    s = self.fread[71430:71455]
    self.equipe2 = s[:s.find('\x00')] # nom équipe 2
    self.OccupTables = unpack('b'*16,self.fread[71455:71471]) # Occupation des tables ( 0: pas attribuée. 1: Eq1. 2: Eq2 )
    self.nbJouInter = unpack('b',self.fread[71471])[0] #nb joueurs par équipe
    # paramètre d'évolution du tableur
    self.NC = unpack('b',self.fread[71472])[0] # N° du coup le plus élevé joué (tirage, jeu et arbitrage)
    #SBInt8("DernierePos"), #__int8 DernierePos; // Dernière position occupée avant la fermeture du tableur
    #ULInt32("LeftCol"), #  int LeftCol; // Dernière position occupée avant la fermeture du tableur
    #ULInt32("TopRow"), #  int TopRow; // Dernière position occupée avant la fermeture du tableur
    self.CoupArbitre = unpack('b',self.fread[71482])[0] #Coup arbitré lors de la fermeture
    self.ArbitrageOk = (unpack('b',self.fread[71483])[0] == 1) # Vrai si tout le tableur est rempli
    #ULInt32("NbLitige"), #  int NbLitige; // Nombre de joueurs en litige (après saisie des cumuls)
    #Array(5,ULInt32("TopPrec")), #  int TopPrec[ 5]; // Tops des 5 parties du TRAP
    self.Terminee = (unpack('b',self.fread[71508])[0] == 1) # Vrai si tout le tableur est rempli
    #il reste des paramètres qui ne m'intéressent pas pour l'instant
    # parsing des coups [16:2791] 
    idx = 16
    i=-1
    for i in range(self.NC):
      self.coups.append(Coup(self.fread[idx:idx+Coup.size],i+1))
      idx += Coup.size
    if i>0:
      self.zetop = self.coups[i].CumulPartie
    else:
      self.zetop=0
    # parsing des joueurs [2791:70491]
    idx = 2791
    for i in range(self.nbtables):
      j = Joueur(self.fread[idx:idx+Joueur.size])
      self.joueursdic[j.table] = j
      idx += Joueur.size
    self.parsed = self.nbcoupsarb()    
    return True

  def close(self):
    """
    Ferme le fichier Duplitop

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.close()
    """
    self.openfile.close()    

  def totals(self):
    """
    Recupère le top et les totaux des joueurs et retourne un tuple
    
    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.totals()
    [1013, 837, 699, 827, 667, 842, 872, 788, 782, 693, 875, 648, 753, 906, 831]
    """
    cp=self.NC
    l = [self.top(cp)]
    return l + [j.tablecumul(cp)[1] for j in self.joueursdic.values() if not j.tablevide()]

  def arbitre(self):
    """
    Retourne le nom de l'arbitre
    
    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.arbitre()
    'HEGGEN Jean-Paul'
     """
    return self.zearbitre
  

  def joueurs(self):
    """
    Retourne une liste de tuples des tables + nom  des joueurs arbitrés (Unicode) + licence
    [(table,nom,licence,serie,club)...]

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.joueurs()[1:3]
    [('48', 'MAES Pierre', 6044187, '7', 'YOD'), ('49', 'LIEGEOIS Anita', 6029105, '5B', 'REC')]
    """
    return [j.joueur() for j in self.joueursdic.values()]

  def joueurscoups(self,cp=None):
    """
    Retourne une liste de tuples des tables + nom (Unicode) +  liste des pts des coups des joueurs arbitrés
    [(table,nom,[coups])...]

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.joueurscoups(5)[1:3]
    [('48', 'MAES Pierre', [22, 0, 68, 104, 53]), ('49', 'LIEGEOIS Anita', [20, 61, 68, 104, 53])]
    """
    if not cp: cp=self.NC;
    return [j.joueurcoups(cp) for j in self.joueursdic.values() if not j.tablevide()]

  def tablescoups(self,cp=None):
    """
    Retourne un tuple table +  liste des tuples pts,avert (a ou p)
    [(table,[[pts,'av'],...])...]

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.tablescoups(3)[1:3]
    [('48', [(22, 'a'), (0, 'z'), (68, '')]), ('49', [(20, ''), (61, ''), (68, '')])]
    """
    if not cp: cp=self.NC;
    return [j.tablecoups(cp) for j in self.joueursdic.values() if not j.tablevide()]

  def tablescoupspts(self,cp=None):
    """
    Retourne un tuple table + tablefixe + liste des pts
    [(table,tablefixe,[pts,...])...]
    
    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.tablescoupspts(3)[1:3]
    [('48', False, [22, 0, 68]), ('49', False, [20, 61, 68])]
    """
    if not cp: cp=self.NC;
    return [j.tablecoupspts(cp) for j in self.joueursdic.values() if not j.tablevide()]

  def partie(self,cp=None):
    """
    Retourne une liste de tuples contentant les coups de la partie du fichier f
    (num du coup,tirage,solution,référence,pts,top)

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.partie()
    [(1, 'AATCEEN', 'ACATENE', 'H4', 74, 74), (2, 'OHR?NAA', 'CHARANgO', '5H', 76, 150), (3, 'GEETN?O', 'ROEnTGEN', 'K5', 82, 232), (4, 'ERIMPAX', 'EXPRIMA', 'I9', 104, 336), (5, 'TEESPBU', 'BUTEES', '16D', 53, 389), (6, 'TIIIGLU', 'LUT', 'G8', 24, 413), (7, 'GIIIDET', 'IDIOTIE', 'O2', 24, 437), (8, 'GDEIIZN', 'DAIGNIEZ', '15H', 95, 532), (9, 'OFIKRVP', 'KIF', 'F7', 45, 577), (10, 'RBVNAOP', 'BRONZA', 'O11', 54, 631), (11, 'PVOUVHS', 'VUS', '9M', 45, 676), (12, 'TPIELLM', 'PLUT', 'N7', 24, 700), (13, 'EILMSWO', 'LIMES', '6B', 28, 728), (14, 'OWDREUJ', 'REDOWA', '4C', 52, 780), (15, 'JUSMNER', 'JE', 'J7', 51, 831), (16, 'MNRSUEY', 'MYE', '11E', 37, 868), (17, 'NRSUAAA', 'RASH', 'I2', 26, 894), (18, 'AANUULO', 'ALUNA', 'C12', 19, 913), (19, 'OUVROCL', 'CRUOR', '2H', 24, 937), (20, 'LOVFEUH', 'LOUVE', 'A2', 43, 980), (21, 'FHQSSE', 'HISSE', '13H', 33, 1013)]
   """
    if not cp: cp=self.NC;
    return [c.partie() for c in self.coups[:cp]]
  
  def nbScrabbles(self):
    return sum([c.scrabble for c in self.coups[:self.NC]])
    
  def top(self,cp=None):
    """
    Retourne le top cumulé au coup cp (actuel par défaut)

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.top(1)
    74
    >>> d.top(10)
    631
    >>> d.top()
    1013    
    """
    if not cp: cp=self.NC;
    return self.coups[cp-1].CumulPartie

  def nbcoups(self):
    """
    Retourne le nombre de coups déjà encodés (int)
    
    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.nbcoups()
    21
    """
    return self.NC

  def tablevide(self,tab):
    """
    Retourne True si la table tab(str) est vide

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.tablevide('56')
    False
    """
    return self.joueursdic[tab].tablevide()

  def nbcoupsarb(self):
    """
    Retourne le nombre de coups arbitrés

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.nbcoupsarb()
    21
    """
    #if self.ArbitrageOk:
     # return self.NC
    #else:
     # return max(0,self.NC-1)
    return min([len([y for y in x[2] if y >-1]) for x in self.joueurscoups()])

  def joucumul(self,cp=None):
    """
    Retourne une liste de tuples des nom + cumul des joueurs arbitrés
    [(table,nom,cumul)...]

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.joucumul(10)[1:3]
    [('48', 'MAES Pierre', 386), ('49', 'LIEGEOIS Anita', 523)]
    """
    if not cp: cp=self.NC;
    return [j.joucumul(cp) for j in self.joueursdic.values() if not j.tablevide()]

  def joucumulneg(self,cp=None):
    """
    Retourne une liste de tuples des nom + cumul négatif des joueurs arbitrés
    [(table,nom,neg)...] où neg =Top, '+10', '-x'

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.joucumulneg(10)[1:4]
    [('48', 'MAES Pierre', '-245'), ('49', 'LIEGEOIS Anita', '-108'), ('46', 'TIXHON Christine', '-264')]
    """
    if not cp: cp=self.NC;
    return [j.joucumulneg(cp,self.top(cp)) for j in self.joueursdic.values() if not j.tablevide()]

  def tablescumul(self,cp=None):
    """
    Retourne une liste de tuples des tables + cumul des joueurs arbitrés
    [('table',cumul)...]

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.tablescumul(10)[1:8]
    [('48', 386), ('49', 523), ('46', 367), ('47', 548), ('44', 510), ('45', 434), ('51', 448)]
    """
    if not cp: cp=self.NC;
    return [j.tablecumul(cp) for j in self.joueursdic.values() if not j.tablevide()]

  def cumultopcoup(self,cp=None):
    """
    !!! redondant avec top
    Retourne le top cumulé au coup cp

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.cumultopcoup()
    1013
    >>> d.cumultopcoup(3)
    232
    >>> d.cumultopcoup(10)
    631
   """
    if not cp: cp=self.NC;
    return self.top(cp)
  def topcoup(self,cp=None):
    """
    Retourne le top au coup cp

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.topcoup(10)
    54
    """
    if not cp: cp=self.parametre.nbcoup;
    return self.coups[cp-1].compte

  def nbjoutop(self,cp=None):
    """
    Retourne le nb de joueurs au top au coup cp

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.nbjoutop()
    0
    >>> d.nbjoutop(1)
    2    
    """
    if not cp: cp=self.NC;
    return len([j for j in self.joucumul(cp) if j[2]==self.top(cp)])

  def fdrtables(self,cp=None):
    """
    retourne une liste de tuples: ("numtable",cumul,[(numcoup,"mot",pts,"neg",av),...])

    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.fdrtables(3)[1:3]
    [('48', 90, [(1, 'ACATENE', 22, '-52', 1), (2, 'CHARANgO', 0, '-76', 30), (3, 'ROEnTGEN', 68, '-14', 0)]), ('49', 149, [(1, 'ACATENE', 20, '-54', 0), (2, 'CHARANgO', 61, '-15', 0), (3, 'ROEnTGEN', 68, '-14', 0)])]
    """
    def f(j,t):
      """
      Retourne un tuple (numcoup,mot,pts,neg,av)
      """
      return (t[0],t[2],j[0],negatize(j[0],t[4]),j[1]) 
    if not cp: cp=self.NC;
    fdrt = [j.fdrtable(cp) for j in self.joueursdic.values() if not j.tablevide()] #tuples ("numtable",top,[(pts,av),...])
    part = self.partie(cp) #tuples (num du coup,tirage,solution,référence,pts,top)
    fdr = []
    for fdj in fdrt:
      l=[]
      for i in range(cp):
        l.append(f(fdj[2][i],part[i]))
      t = fdj[0],fdj[1],l
      fdr.append(t)
    return fdr

  def backup(self,path):
    """ 
    copie son fichier dans le folder path\self.nom-self.path
    Retourne True si ok
    """
    from os.path import join as pjoin
    from os.path import basename
    from shutil import copyfile
    dst = pjoin(path ,self.nom + '-' + basename(self.fichier))
    try:
      copyfile(self.fichier,dst)
      return True
    except IOError:
      return False

  def showpartie(self,cp=None):
    """
    Imprime la partie à l'écran
    """
    if not cp: cp=self.NC;
    for c in self.coups[:cp]:
      c.show()

  def showclassement(self,cp=None):
    """
    Imprime le classement à l'écran
    """
    if not cp: cp=self.NC;
    jl = self.joucumul(cp)
    jl.sort(key = lambda x: x[2],reverse = True)
    plcrun = 1
    ptold = 0
    plc =0
    sz = "PLC " + "  JOUEUR                           " + "  PTS"
    print sz
    for table,jou,pts in jl:
      if pts != ptold:
        ptold = pts
        plc = plcrun
      sz = str(plc).ljust(6," ") + jou.ljust(33, " ") + str(pts).rjust(5," ")
      print sz
      plcrun += 1

  def showscrabble(self,cp=None):# todo
    """
    Imprime ce qui pourraient être les scrabbles à l'écran
    """
    if not cp: cp=self.NC;
    ls = [c for c in self.coups[:cp] if c.scrabble == True]
    if len(ls)>0:
      print "Scrabble(s) de la partie"
      for c in ls:
        c.show() 

  def showsolo(self):# todo
    """
    Imprime les solos à l'écran
    """
    pass
  
  def showfdr(self):
    """
    Imprime les feuilles de route à l'écran
    """
    print "*****     FEUILLES DE ROUTE     *****"
    for fdr in self.fdrtables():
      print "Table %s: %d" % (fdr[0],fdr[1])
      sz =  "CP" + "|" + "      MOT      " + "|" + "PTS" + "|" + " NEG" + "|" + "AV" + "|"
      print sz
      print "=" * len(sz)
      for cp in fdr[2]:
        sz = str(cp[0]).ljust(2," ") + "|" + cp[1].center(15," ") + "|" + str(cp[2]).rjust(3," ") + "|" + cp[3].rjust(4," ") + "|" + str(cp[4]).rjust(2," ") + "|"
        print sz
      
  def joueur(self,t):
    """
    Retourne le joueur dont la table est t sous forme str
    
    >>> d=Duplitop('test','test.hop')
    >>> d.open()
    True
    >>> d.read()
    True
    >>> d.parsefile()
    True
    >>> d.joueur('56')
    56 - LEJEUNE Chantal - YOD - 5B - - - BE
    837 : 22 - 22 - 66 - 104 - 53 - 24 - 24 - 94 - 35 - 54 - 45 - 24 - 28 - 28 - 51 - 37 - 14 - 17 - 19 - 43 - 33
    """
    return self.joueursdic[t]
  
  def find_solo(self,cp,ljou):
    """
    Cherche un solo au coup cp parmi les joueurs de ljou et retourne le joueur et les pts du solo
    """
    if len(ljou)==0: return None
    if not cp: cp=self.NC;
    l=[j.ptscoup(cp) for j in ljou]
    lm=max(l)
    if l.count(lm) == 1:
      return [ljou[l.index(lm)],lm]
    else:
      return None
    
  def tables(self):
    return sorted([int(k) for k in self.joueursdic.keys()])
    
class Case(object):
  """
  Case de grille
  """
  def __init__(self,type='l1',char=''):
    self.char=char # caractère de la case
    self.type=type # coin,rep,l1, l2, l3, m2, m3, jok
    
  def set_char(self,c):
    """
    Set le caractère de la case
    """
    self.char = c
    if c.islower():
      self.type='jok'
    
def transpref(ref):
  """
  Retourne la référence transposée
  """
  r1 = ref[0]
  r2=''
  for c in ref[1:]:
    if r1.isalpha():
      r2 += c
    else:
      if c.isdigit():
        r1 += c
      else:
        r2 += c
  return r2+r1
  
class Grille(object):
  """
  La grille est 'vue' comme un tableau de 17x17
  et comporte les repères alphanum des 4 côtés.
  En fait, c'est un dico dont le clefs sont les coordonnées dans la grille
  et les valeurs sont des cases avec chaque case jouable atteignable par une
  coordonnée et sa transposée.
  """
  def __init__(self):
    rg=[]
    for i in range (1,16):# construit la liste des réf jouables possibles de la grille
      for j in range(1,16):
        rg.append('%s%d' % (chr(64+i),j))
    self.board={}
    self.board['@0'] = Case('coin') #coin supérieur gauche
    self.board['P0'] = Case('coin') # coin inférieur gauche
    for i in range(1,16):# ligne de repères num
      self.board['@%d' % i] = (Case('rep','%d' % i))
      self.board['P%d' % i] = (Case('rep','%d' % i))
    self.board['@16'] = Case('coin') #coin supérieur droit
    self.board['P16'] = Case('coin') #coin inférieur droit
    for i in range(15): # ligne de repères alpha
      self.board['%s0' % chr(65+i)] = Case('rep',chr(65+i))
      self.board['%s16' % chr(65+i)] = Case('rep',chr(65+i))
    for r in ('A1','A8','A15','H1','H15','O1','O8','O15'):# cases mot x 3
      self.board[r]=self.board[transpref(r)]=Case('m3')
      rg.remove(r)
    for r in ('B2','B14','C3','C13','D4','D12','E5','E11','N2','N14','M3','M13','L4','L12','K5','K11','H8'):# cases mot x 2
      self.board[r]=self.board[transpref(r)]=Case('m2')
      rg.remove(r)
    for r in ('B6','B10','F2','F6','F10','F14','N6','N10','J2','J6','J10','J14'):# cases lettre x 3
      self.board[r]=self.board[transpref(r)]=Case('l3')
      rg.remove(r)
    for r in ('A4','A12','C7','C9','D1','D8','D15','G3','G7','G9','G13','H4','H12','I3','I7','I9','I13','L1','L8','L15','O4','O12'):# cases lettre x 2
      self.board[r]=self.board[transpref(r)]=Case('l2')
      rg.remove(r)
    for r in rg:
      self.board[r]=self.board[transpref(r)]=Case('l1')
  
  def setPartie(self,p):
    """
    Récupère une partie sous la forme (num du coup,tirage,solution,référence,pts,top)
    """
    self.partie = p
    for c in p:
      if c[3][0].isalpha(): # mot horizontal
        base = c[3][0]
        idx = int(c[3][1:])
        for l in c[2]:
          self.board['%s%d' % (base,idx)].set_char(l)
          idx += 1
      else: # mot vertical
        if c[3][1].isdigit():
          base = c[3][:2]
          idx = ord(c[3][2])
        else:
          base = c[3][0]
          idx = ord(c[3][1])
        for l in c[2]:
          self.board['%s%s' % (base,chr(idx))].set_char(l)
          idx += 1
    return
          
  def tolol(self):
    """
    Retourne la grille remplie sous forme de liste de listes
    """
    ll=[]
    for i in range(64,81):#de @ à P
      l = []
      for j in range(17):
        l.append(self.board['%s%d' % (chr(i),j)])
      ll.append(l)
    return ll

def main():
  d=Duplitop('test','ice5.inp')
  d.open()
  d.read()
  d.parsefile()
  l=d.fdrtables(len(d.partie()))
  l.sort(key=lambda x: int(x[0]))
  for j in l:
    print "Table %s: %d" %(j[0],j[1])
    for c in j[2]:
      print "\t",c
  
def maintest():
  d=Duplitop('test','ice5.inp')
  d.open()
  d.read()
  d.parsefile()
  d.showpartie()
  print "Nbs:",d.nbScrabbles()

if __name__ == "__main__":
  #main()
  maintest()
#  _test()
  print "Tests done"
