# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
from datetime import datetime, timedelta

from com.sun.star.uno import RuntimeException
from com.sun.star.lang import IllegalArgumentException
from com.sun.star.util import DateTime

import utils


class Inscriptions(utils.Tournoi):
    HLIGNE = 40   # hauteur des lignes dans le permutateur
    arrayinscrits = []
    def __init__(self):
        super(Inscriptions, self).__init__()

    def ouvrirInscrits(self,source):
        '''Ouvre le classeur Inscrits'''
        if utils.databasedocument.isModified(): utils.databasedocument.store()
        form = source.Model.Parent
        nbInscrits = form.getShort(form.findColumn("Licences"))
        if not nbInscrits:
            self.msgbox("Il n'y a encore aucun inscrit !\t", "Erreur", 'warn')
            return
        # ouverture proprement dite
        nouvURL = self.convertToUrl('{tooournoiPath}{nom}/{nom}_Inscrits.ods'.format(**self.infos))
        try:
            classeur = self.loadComponentFromURL(nouvURL)
        except (RuntimeException, IllegalArgumentException):
            modele = self.convertToUrl( self.infos['extPath'] + "modele_Inscrits.ots")
            classeur = self.loadComponentFromURL(modele)
            classeur.storeAsURL( nouvURL, () )
        # hack pour éviter une erreur si classeur sauvegardé en mode Aperçu :
        ctrl = classeur.getCurrentController()
        frame = ctrl.getFrame()
        frame.ComponentWindow.setFocus()
        if not self.hasUnoInterface(ctrl, "com.sun.star.sheet.XSpreadsheetView"):
            self.executeDispatch(".uno:PrintPreview", frame=frame)
        # mise à jour des plages sources
        dbR = classeur.DatabaseRanges
        nR = classeur.NamedRanges
        nR.getByName("joueur2").ReferredCells.Columns.IsVisible = self.infos['paires']
        f = classeur.Sheets.getByIndex(0)
        classeur.lockControllers()
        try:
            for db in self.iterUno(dbR):
                db.refresh()
            # définir les plages d'impression
            pA = f.PrintAreas
            pA[0].EndRow = dbR.getByName("ImportINSCRITS").DataArea.EndRow
            pA[0].EndColumn = dbR.getByName("ImportINSCRITS").DataArea.EndColumn
            f.setPrintAreas( (pA[0],) )
        finally:
            classeur.unlockControllers()
            classeur.store()

    def ouvertureInscription(self, source):
        '''Routine exécutée au chargement du formulaire Inscriptions'''
        # vérification de la validité de la base Joueurs
        cles = self.clesOptions()
        derdate =  datetime.strptime(cles.PlayersUpdateDate, '%Y%m%d')
        if (datetime.now() - derdate).days > 31:
            self.msgbox('ATTENTION !\n\n'
                        'La base de données "Joueurs" a plus d\'un mois.\t\n'
                        'Il est vivement conseillé de la mettre à jour.\n\n'
                        'AVERTISSEMENT', 'warn')
            # prévenir à nouveau dans 10 jours
            cles.setPropertyValue("PlayersUpdateDate", datetime.strftime(datetime.now()-timedelta(21), '%Y%m%d'))
            cles.commitChanges()

    def inscrireJoueur(self, source):
        '''Inscrit le joueur sélectionné'''
        controller = self.doc.CurrentController
        form = source.Model.Parent
        joueurs = form.Parent.getByName("joueurs")
        selecteur1 = controller.getControl(form.getByName("selecteur1"))
        if self.infos['paires']:
            selecteur2 = controller.getControl(form.getByName("selecteur2"))
        if joueurs.isAfterLast(): return    #' aucun joueur sélectionné
        inscrits = form.Parent.getByName("inscrits")
        if not self.infos['paires'] or selecteur1.Text:
            licence = joueurs.getString(2)
            if source.Model.Name == "btnInscrireJoueur":    # déclenché par bouton INSCRIRE
                self.execute('INSERT INTO "T_Parties" ("Licence") VALUES (%s)' % licence)
            else:   # déclenché par bouton "MODIFIER"
                licenceactuelle = inscrits.getString(2)
                self.execute('UPDATE "T_Parties" SET "Licence" = %s WHERE "Licence" = %s' % (licence, licenceactuelle))
            if self.infos['paires'] and selecteur1.Text: selecteur2.setFocus()
        else :
            licence = joueurs.getString(2)
            licenceactuelle = inscrits.getString(2)
            self.execute('UPDATE "T_Parties" SET "Licence2" = %s WHERE "Licence" = %s' % (licence, licenceactuelle))
            selecteur1.setFocus()
        if self.infos['paires']:
            inscrits.Order = '"Licence2" IS NOT NULL, "DateInscription" DESC'
        else:
            inscrits.Order = '"DateInscription" DESC'
        inscrits.ApplyFilter = False
        inscrits.reload()
        inscrits.ApplyFilter = True
        if self.infos['paires']:
            inscrits.beforeFirst()
            while inscrits.next():
                if inscrits.getString(2) == licence or inscrits.getString(4) == licence:
                    break
            form.getByName("selecteur2").reset()
        form.getByName("selecteur1").reset()

    def filtreJoueurs(self, source):
        '''Filtre les joueurs suivant la saisie (formulaire inscriptions solo)'''
        filtre = source.Text or "zzz"
        if self.hasUnoInterface(source, "com.sun.star.awt.XControl"):
            form_joueurs = source.Model.Parent.Parent.getByName("joueurs")
        else:
            form_joueurs = source.Parent.Parent.getByName("joueurs")
        command = ('SELECT "Nom" || \' \' || "Prénom" AS "Joueur", "Licence",\n'
                   '"Série", "Cat", "Club", "Nationalité" FROM "T_Joueurs"\n'
                   '%s\n'
                   'AND "Licence" NOT IN (SELECT "Licence" FROM "T_Parties")\n'
                   'AND "Licence" NOT IN (SELECT "Licence2" FROM "T_Parties")\n'
                   'ORDER BY "Nom", "Prénom"')
        if filtre.isdigit():
               where = 'WHERE "Licence" LIKE \'%s%%\'\n' % filtre
        else:
               where = 'WHERE LCASE("Joueur") LIKE \'%s%%\'\n' % filtre.replace("'","''").lower()
        form_joueurs.Command = command % where
        form_joueurs.reload()

    def verifDoublonLicence(self, source):
        '''Vérifie les doublons depuis les champs Licence'''
        form = self.doc.DrawPage.Forms.getByName('Standard')
        licence = source.Text.strip()
        if licence == source.BoundField.String: # si pas de modif
            return
        elif not licence:  # si aucune saisie ou si effacement
            source.BoundField.updateNull()
            form.reset()
            return
        elif len(licence) < 7: # si longueur licence non valide
            self.msgbox("Ce numéro de licence n'est pas valable.\t", "Avertissement", 'warn' )
            source.BoundField.updateNull()
            form.reset()
            return
        joueur = self.nomDuJoueur( licence )
        if joueur == '':  # si le joueur est inconnu
            # on souhaite peut-être ajouter ce joueur...
            if self.msgbox("Ce joueur n'existe pas dans la base. Voulez-vous l'ajouter ?\t",
                            "Joueur inconnu", 'query', ('yesno',)) == 2:
                self.ouvreJoueur(licence)    # routine ouvrant le formulaire de modification de la base Joueurs
            # ... ou peut-être pas
            else:
                if form.IsNew:
                    source.BoundField.updateNull()
                else:
                    form.cancelRowUpdates()
                form.reset()
        elif self.estInscrit(licence):  # routine 'estInscrit' plus bas
            self.msgbox( "%s est déjà inscrit(e).\t" % joueur, "Avertissement", 'warn' )
            if form.IsNew:
                source.BoundField.updateNull()
            else:
                form.cancelRowUpdates()
            form.reset()
        self.reloadMenu()

    def verifDoublonJoueur(self, source):
        '''Vérifie les doublons depuis les champs Joueur'''
        form = self.doc.DrawPage.Forms.getByName('Standard')
        if not form.RowCount: return
        # self.doc.CurrentController.getControl(form.TabInscriptions ).commit()
        licence = source.ValueItemList[source.SelectedItems[0]]
        if self.estInscrit(licence):
            joueur = self.nomDuJoueur(licence)
            self.msgbox( "%s est déjà inscrit(e)." % joueur, "Avertissement", 'warn')
            if form.IsNew:
                source.BoundField.updateNull()
            else:
                form.cancelRowUpdates()
            form.reset()
        self.reloadMenu()

    def verifDoublonTable(self, source):
        '''Vérifie les doublons depuis le champ Table'''
        form = self.doc.DrawPage.Forms.getByName('inscrits')
        table = source.CurrentValue
        if not form.RowCount:
            return
        elif table == source.BoundField.Value or not source.Value:
            return
        elif source.Value == 0:
            self.msgbox("Pas de table 0, merci.\t", "Erreur", 'warn' )
            source.BoundField.updateNull()
            form.reset()
            return
        if self.infos['tablesparalleles']:
            estFixe = form.getBoolean(form.findColumn("Fixe"))
            filtre = '"Fixe" = %s AND "TableEnCours" = %d' % (estFixe, table)
        else:
            filtre = '"TableEnCours" = %s' % table
        rowset = self.genereRowSet( 0, "T_Parties", filtre )
        rowset.execute()
        if not rowset.RowCount:
            x = self.infos['derpartie']
            form.updateString( form.findColumn( "TableEnCours" ), table )
            if x > 0:
                self.execute('UPDATE "T_Parties" SET "T%d" = %d WHERE "ID_Partie" = %s' % (x, table, form.getString(form.findColumn("ID_Partie"))))
            if form.getBoolean(form.findColumn( "Fixe" )): form.updateString( form.findColumn( "TableProvisoire" ), table )
        else:
            self.msgbox("La table n° %d est déjà attribuée." % table, "Avertissement", 'warn')
            form.reset()
        rowset.dispose()

    def ouvreJoueur(self, licence=''):
        '''Ouvre le formulaire d'édition de la base Joueurs'''
        formdoc = self.ouvreForm("F_Joueurs")
        form = formdoc.DrawPage.Forms.getByName('MainForm')   # la couche formulaire
        form.load()  # indispensable pour la consistance de l'objet
        form.last()  # se positionne sur le dernier (résout affichage !)
        form.moveToInsertRow()  # se positionne sur nouvel enregistrement
        if licence:
            form.TableControl.getByName('colLicence').Text = licence
            form.TableControl.getByName('colLicence').commit()
            formdoc.Component.CurrentController.getControl(form.TableControl).setCurrentColumnPosition(0)

    def reclasserPaires(self, source):
        '''Classe les paires alphabétiquement'''
        form = source.Model.Parent
        inscrits = self.executeQuery('SELECT COUNT("ID_Partie") FROM "T_Parties" WHERE "ID_Partie" > 0')
        if not inscrits:
            self.msgbox("Il n'y a encore aucune paire inscrite !","Avertissement",'warn')
            return
        sql = ('UPDATE "T_Parties" AS "P" SET "Licence" = "Licence2", "Licence2" = "Licence" '
               'WHERE (SELECT "Nom" || "Prénom" FROM "T_Joueurs" WHERE "Licence" = "P"."Licence") > '
               '(SELECT "Nom" || "Prénom" FROM "T_Joueurs" WHERE "Licence" = "P"."Licence2")')
        self.execute(sql)
        form.reload()
        # form.last()    # règle problème d'affichage

    def autoriseNouveauxJoueurs(self, source):
        '''Débloque l'inscription directe'''
        bdp = self.barreDeProgression('Patientez, chargement de la base "Joueurs"...  ')
        bdp.start()
        form = source.Model.Parent
        tabI = form.getByName("TabInscriptions")
        bdp.setValue(50)
        if self.infos['paires']:
            tabI.getByName("ColJoueur2").Enabled = True
            tabI.getByName("ColLicence2").Enabled = True
        bdp.setValue(75)
        tabI.getByName("ColJoueur1").Enabled = True
        tabI.getByName("ColLicence1").Enabled = True
        form.last()
        form.moveToInsertRow()
        source.setVisible(False)
        form.getByName("lblInfos").EnableVisible = True
        form.getByName("ouvreJoueurs").EnableVisible = True
        bdp.end()

    def reloadMenu(self):
        '''Met à jour les données du menu Inscriptions'''
        menu = utils.databasedocument.FormDocuments.getByName("Menu_Principal").Component
        menu.DrawPage.Forms.getByName("Standard_Menu_Inscriptions").reload()
        menu.DrawPage.Forms.getByName("Total").reload()

    def permutateur(self):
        '''Permute les tables de deux joueurs en conservant leurs points'''
        # 2 couches formulaires pour une même table
        forms = (self.doc.DrawPage.Forms.getByName("Standard1"),
                 self.doc.DrawPage.Forms.getByName("Standard2"))
        bmks = (forms[0].Bookmark, forms[1].Bookmark)
        # vérification élémentaire
        if forms[0].Row == forms[1].Row:
            self.msgbox("\nBesoin de repos ?\n\n"
                        "Tu essaies de permuter un joueur avec lui-même...\t\n\t",
                        "Avertissement", 'warn')
            return
        # récupération des valeurs à permuter
        avecscore = forms[0].getByName("AvecScore").State
        colLic = forms[0].findColumn("Licence")
        dictpermut = {'p': self.infos['derpartie'] or 1,
                      'l1': forms[0].getString(colLic),
                      'l2': forms[1].getString(colLic)}
        coltable = ('T%(p)s' % dictpermut, 'TableEnCours')
        # permutation proprement dite
        if self.infos['tablesaumerite']:
            colFixe = forms[0].findColumn("Fixe")
            if forms[0].getBoolean(colFixe) and forms[1].getBoolean(colFixe):
                coltable += ('TableProvisoire',)
            elif forms[0].getBoolean(colFixe) != forms[1].getBoolean(colFixe):
                if self.infos['tablesparalleles']:
                    self.msgbox("Il n'est pas possible de permuter entre\t\n"
                                "une table fixe et une table normale.",
                                "Erreur", 'warn')
                    return
        SQL = 'UPDATE "T_Parties" SET\n'
        SET = []
        for t in coltable:
            dictpermut.update(t=t)
            SET.append(' "%(t)s" = (SELECT SUM("%(t)s") FROM "T_Parties"\n'
                       '            WHERE "Licence" IN (%(l1)s, %(l2)s)) - "%(t)s"' % dictpermut)
        SQL += ',\n'.join(SET)
        if avecscore:
            SQL += (',\n'
                    ' "P%(p)s" = (SELECT SUM("P%(p)s") FROM "T_Parties"\n'
                    '            WHERE "Licence" IN (%(l1)s, %(l2)s)) - "P%(p)s",\n'
                    ' "R%(p)s" = (SELECT SUM("R%(p)s") FROM "T_Parties"\n'
                    '            WHERE "Licence" IN (%(l1)s, %(l2)s)) - "R%(p)s"' % dictpermut)
        SQL += '\nWHERE "Licence" IN (%(l1)s, %(l2)s);' % dictpermut
        # self.callXray(SQL)
        # return
        self.executeQuery(SQL)
        if avecscore:
            from parties import Parties
            P = Parties()
            P.recalculCumul()
            P.xTableProvisoire(dictpermut['p'])
        # mettre à jour l'affichage
        for form, bmk in zip(forms,bmks):
            form.getByName('tblPermut').RowHeight = self.HLIGNE   # garder l'équilibre entre les deux tableaux
            form.reload()
            form.afterLast()
            form.moveToBookmark(bmk)

    def tabulateur(self, source):
        '''Lance l'attribution automatique des numéros de table'''
        from time import sleep

        def gettablesfixes():
            tablesfixes = []
            liste = dlg.getControl('fixe4').Text
            try:
                for s in liste.replace(' ','').split(','):
                    if '-' in s:
                        a,b = (int(x) for x in s.split('-'))
                        tablesfixes += range(a,b+1)
                    else:
                        tablesfixes.append(int(s))
            except ValueError:
                return None
            else: return set(tablesfixes)

        dlg = source.Context
        dlg.endExecute()
        PREMIERESTABLESFIXES = dlg.getControl('fixe1').Model.State
        DERNIERESTABLESFIXES = dlg.getControl('fixe2').Model.State
        TABLESFIXESSPECIFIQUES = dlg.getControl('fixe3').Model.State
        rows, rowsfixes, updatequery = [], [], ''
        # RÉCUPÉRER LES JOUEURS
        # on récupère les joueurs dans l'ordre défini par le mode d'attribution
        mode = source.Model.Name
        modes = {'parclassement':'"serie" ASC, "classement" ASC',
                 'parclassement2':'"serie" ASC, "classement" ASC',
                 'pardate':'"DateInscription" ASC',
                 'pardate2':'"DateInscription" ASC',
                 'clubsnoncontigus':'"DateInscription" ASC',
                 'clubsnoncontigus2':'"DateInscription" ASC'}
        SQL = ( 'SELECT "ID_Partie", "Club", "Fixe", "DateInscription",\n'
                '       IFNULL(COALESCE( CASEWHEN("b"."Classement" < "c"."Classement",\n'
                '            "b"."Classement", "c"."Classement"), "b"."Classement"),999999) AS "classement",\n'
                '       COALESCE( CASEWHEN("b"."Série" < "c"."Série",\n'
                '            "b"."Série", "c"."Série"), "b"."Série") AS "serie"\n'
                'FROM "T_Parties" AS "a"\n'
                '     LEFT OUTER JOIN "T_Joueurs" AS "b" ON "a"."Licence" = "b"."Licence"\n'
                '     LEFT OUTER JOIN "T_Joueurs" AS "c" ON "a"."Licence2" = "c"."Licence"\n'
                'WHERE "a"."ID_Partie" > 0 AND "Fixe" = %s\n'
                'ORDER BY %s')
        # on récupère d'abord les joueurs fixes
        rowset = self.executeQuery(SQL % (True, modes[mode]))
        while rowset.next():
            rowsfixes.append((rowset.getInt(1),))   # on se contente de l'ID
        # on récupère ensuite les joueurs non fixes
        rowset = self.executeQuery(SQL % (False, modes[mode]))
        while rowset.next():
            rows.append((rowset.getInt(1), rowset.getString(2)))    # ID + club, pour l'option 'clubs non contigus'
        if 'clubsnoncontigus' in mode:
            from random import randrange
            n=1
            while n < len(rows):
                if rows[n][1] == rows[n-1][1]:
                    npermut = randrange(len(rows))
                    rows[n], rows[npermut] = rows[npermut], rows[n]
                    n=0
                else: n+=1
        # ATTRIBUTION PROPREMENT DITE
        bdp = self.barreDeProgression("Attribution des tables...", len(rows)+len(rowsfixes))
        bdp.start()
        # tables fixes en parallèle
        if self.infos['tablesparalleles']:
            for n, row in enumerate(rowsfixes, 1):
                updatequery += ('UPDATE "T_Parties" SET "TableEnCours" = %d WHERE "ID_Partie" = %s;\n'
                                    % (n, row[0]))
                bdp.setValue(n)
            for n, row in enumerate(rows, 1):
                updatequery += ('UPDATE "T_Parties" SET "TableEnCours" = %d WHERE "ID_Partie" = %s;\n'
                                    % (n, row[0]))
                bdp.setValue(n+len(rowsfixes))
        else:
            # tables fixes = dernières tables
            if DERNIERESTABLESFIXES:
                rows += rowsfixes
                for n, row in enumerate(rows, 1):
                    updatequery += ('UPDATE "T_Parties" SET "TableEnCours" = %d WHERE "ID_Partie" = %s;\n'
                                        % (n, row[0]))
                    bdp.setValue(n)
            # tables fixes spécifiques
            elif TABLESFIXESSPECIFIQUES and rowsfixes:
                tablesfixes = gettablesfixes()
                if not tablesfixes:
                    self.msgbox('Les tables fixes fournies sont incorrectes.\t', 'Erreur', 'warn')
                    rowset.dispose()
                    bdp.end()
                    return
                tables = set(range(1, len(rows)+len(rowsfixes)+1))
                tables -= tablesfixes   # une table ne peut se retrouver dans les deux sets
                tablesfixes = iter(sorted(tablesfixes)) # plus confortable avec un générateur
                for n, row in enumerate(rowsfixes, 1):
                    try:
                        updatequery += ('UPDATE "T_Parties" SET "TableEnCours" = %d WHERE "ID_Partie" = %s;\n'
                                            % (next(tablesfixes), row[0]))
                        bdp.setValue(n)
                    except StopIteration:
                        self.msgbox('Les tables fixes fournies sont insuffisantes.\n'
                                    'L\'attribution des tables fixes sera en partie incorrecte.\t', 'Erreur', 'warn')
                        rows = rowsfixes[n-1:]+rows   # si trop peu de tables réservées, on récupère les joueurs fixes excédentaires
                        break
                tables = iter(sorted(tables | set(tablesfixes)))  # si trop de tables réservées, on récupère celles non attribuées
                for m, row in enumerate(rows, 1):
                    updatequery += ('UPDATE "T_Parties" SET "TableEnCours" = %d WHERE "ID_Partie" = %s;\n'
                                        % (next(tables), row[0]))
                    bdp.setValue(m+n-1)
            else:
            # pas de tables fixes ou tables fixes = premières tables
                rows = rowsfixes+rows
                for n, row in enumerate(rows, 1):
                    updatequery += ('UPDATE "T_Parties" SET "TableEnCours" = %d WHERE "ID_Partie" = %s;\n'
                                        % (n, row[0]))
                    bdp.setValue(n)
        self.executeQuery(updatequery)
        bdp.setText('Opération réussie\t')
        sleep(0.5)
        bdp.end()

    def controleSaisieTablesFixes(self, source):
        texte = source.Text
        charOK = "0123456789-,"
        texte = ''.join(x for x in texte if x in charOK)
        if texte:
            while texte[0] not in charOK[:-2]: texte = texte[1:]
        if texte:
            while texte[-1] not in charOK[:-2]: texte = texte[:-1]
        source.setText(texte)

    def choixFichierInscrits(self, source):
        '''Associé à l'événement 'texte modifié' du contrôle FileControl,
        vérifie la validité du fichier et récupère les en-têtes de colonnes'''
        dlg = source.Context
        # charge les données dans Calc
        url = self.convertToUrl(dlg.getControl("FileControl").Text)
        doc = self.importViaCalc(url, Hidden=True)
        if not doc:    # url inexistante -> on ignore
            pass
        else:
            # on récupère les données
            del self.arrayinscrits[:]
            try:
                f = doc.CurrentController.ActiveSheet
                curseur = f.createCursor()
            except AttributeError:
                self.msgbox("Ce type de fichier n'est pas pris en charge.\t", "Erreur", 'warn')
                return
            else:
                curseur.gotoStartOfUsedArea(False)
                curseur.gotoEndOfUsedArea(True)
                self.arrayinscrits.append(curseur.getDataArray())
                # active les listbox et les alimente avec les valeurs de la deuxième ligne comme exemple
                for n in range(1,7):
                    lbox = dlg.getControl("boxEnTete%d" % n)
                    lbox.removeItems(0, lbox.ItemCount )    # efface si une première sélection à remplacer
                    if (n != 2 or self.infos['paires']):
                        lbox.setEnable(True)    # n'active Licence2 que pour parties par paires
                    lbox.addItems(self.arrayinscrits[0][1],0)
                    lbox.DropDownLineCount = lbox.ItemCount #cosmétique
            finally:
                doc.close(True)

    def importinscrits(self, source):
        dlg = self.createTooournoiDialog('dialogue4')
        dlg.getControl("lblLicence2").Enable = self.infos['paires']
        while 1:
            if not dlg.execute(): return
            # vérifie que le champ licence au minimum est fourni
            if dlg.getControl("boxEnTete1").SelectedItemPos < 0:
                self.msgbox("Le champ 'Licence' est obligatoire.\t", "Erreur", 'warn')
            else:
                break
        # la liste des inscrits est dans une variable de classe
        inscrits = self.arrayinscrits.pop()
        # on récupère les index de colonne (colonnesPos défini dans utils)
        colLic, colLic2, colTable, colFixe, colDate, colPaye = utils.colonnesPos
        # on efface les données existante si demandé
        if dlg.getControl("optEcraser").State:
            self.executeQuery('DELETE FROM "T_Parties" WHERE "ID_Partie" > 0')
            self.executeQuery('ALTER TABLE "T_Parties" ALTER COLUMN "ID_Partie" RESTART WITH 1')
        # on vérifie les différentes valeurs à importer
        bdp = self.barreDeProgression("Importation des joueurs, patientez...", len(inscrits))
        bdp.start()
        rowset = self.genereRowSet(0,"T_Parties")
        rowset.execute()
        absentsbase, dejainscrits = [], []
        sql = ''
        for n, inscrit in enumerate(inscrits, start=1):
            licOK, lic2OK = True, False
            joueur = ''
            # on vérifie la première licence
            try:
                licence = str(int(inscrit[colLic]))
            except ValueError:
                licence = ''
            if len(licence) != 7:  # évite erreur si mauvaise licence
                if n > 1:   # 1 = sûrement ligne d'en-tête, on ne logue pas
                    absentsbase.append("Ligne %d. %s" % (n, licence if len(licence) else '?'))
                licOK = False
            else:
                joueur = self.nomDuJoueur(licence)
                # on s'assure que le joueur est repris dans la base
                if joueur:
                    # on vérifie que le joueur n'est pas déjà inscrit
                    if self.estInscrit(licence):
                        dejainscrits.append("Ligne %d.\t%s\t(%s)" % (n, joueur, licence))
                        licOK = False
                else:
                    # on récupère la licence
                    absentsbase.append("Ligne %d. %s" %(n, licence if len(licence) else '?'))
                    licOK = False
            # on vérifie la deuxième licence si nécessaire
            if self.infos['paires'] and colLic2 > -1:
                try:
                    licence2 = str(int(inscrit[colLic2]))
                except ValueError:
                    licence2 = ''
                if len(licence2) != 7:
                    if n > 1:   # 1 = sûrement ligne d'en-tête, on ne logue pas
                        absentsbase.append("Ligne %d. %s" %(n, licence2 if len(licence2) else '?'))
                else:
                    joueur = self.nomDuJoueur(licence2)
                    if joueur:
                        if self.estInscrit(licence2):
                            dejainscrits.append("Ligne %d.\t%s\t(%s)" % (n, joueur, licence2))
                        else:
                            lic2OK = True
                    else:
                        absentsbase.append("Ligne %d. %s" %(n, licence2 if len(licence2) else '?'))
            # si joueur valide, insertion de données de l'enregistrement
            if licOK or lic2OK:
                insertsql = ('INSERT INTO "T_Parties" ("Licence","Licence2","TableEnCours","DateInscription","Fixe","Payé","Infos") '
                             'VALUES ({Licence},{Licence2},{TableEnCours},{DateInscription},{Fixe},{Paye},0);\n')
                values = {'Licence2':'NULL', 'TableEnCours':'NULL', 'DateInscription':'NULL', 'Fixe':'False', 'Paye':'False'}
                # la ou les licences correctes
                if licOK:
                    values['Licence'] = licence
                    if lic2OK:
                        values['Licence2'] = licence2
                # si licence 1 non valide, insère la 2 à la place car colonne Licence est requise
                else:   #(rappel : on n'arrive pas jusqu'ici si aucune n'est correcte)
                    values['Licence'] = licence2
                # la date si fournie
                if colDate > -1:
                    date = self.calcdate2datetime(inscrit[colDate])
                    values['DateInscription'] = "'%s'" % date.isoformat(' ')
                else: values['DateInscription'] = 'CURRENT_TIMESTAMP'
                # table si fournie
                # !!! AJOUTER UN CONTRÔLE POUR ÉVITER DOUBLETS (triplets si numérotation parallèle) !!!
                if colTable > -1:
                    try:
                        table = int(inscrit[colTable])
                        if table > 0:
                            values['TableEnCours'] = table
                    except ValueError:
                        pass
                # fixe si fourni
                if colFixe > -1:
                    try:
                        fixe = str(inscrit[colFixe]).upper()
                    except UnicodeEncodeError:
                        fixe = unicode(inscrit[colFixe]).upper()
                    if fixe not in ("NON","NO","FAUX","FALSE","0",""):
                        values['Fixe'] = 'True'
                # payé si fourni
                if colPaye > -1:
                    try:
                        paye = str(int(inscrit[colPaye]))
                    except ValueError:
                        try:
                            paye = str(inscrit[colPaye]).upper()
                        except UnicodeEncodeError:
                            paye = unicode(inscrit[colPaye]).upper()
                    if paye not in ("NON","NO","FAUX","FALSE","0",""):
                        values['Paye'] = 'True'
                # mémorise l'enregistrement
                sql += insertsql.format(**values)
            bdp.setValue(n)
        if sql: self.executeQuery(sql)
        utils.databasedocument.store()
        # logs
        texteLog = []
        if dejainscrits:
            if len(dejainscrits) == 1:
                texte = "Le joueur suivant est déjà inscrit\net n'a donc pas été ajouté :\n\n"
            else:
                texte = "Les joueurs suivants sont déjà inscrits\net n'ont donc pas été ajoutés :\n\n"
            texteLog.append(texte + '\n'.join(dejainscrits))
        if absentsbase:
            if len(absentsbase) == 1:
                texte = ("La licence suivante est inconnue.\n"
                         "Il faudra ajouter ce joueur à la\nbase avant de pouvoir l'inscrire :\n\n")
            else:
                texte = ("Les licences suivantes sont inconnues.\n"
                         "Il faudra ajouter ces joueurs à la\nbase avant de pouvoir les inscrire :\n\n")
            texteLog.append(texte + '\n'.join(absentsbase))
        rowset.dispose()
        bdp.end()
        dlg.dispose()
        source.Model.Parent.reload()
        # dialogue de log
        if texteLog:
            dlgLog = self.createTooournoiDialog('dialogue6')
            texteCtrl = dlgLog.getControl("bTexte")
            texteCtrl.setText( ('\n'*2 + '-'*15 + '\n'*2).join(texteLog) )
            dlgLog.execute()
            dlgLog.dispose()

    def importinscrits2(self, source):
        """
        Permet la modification des inscrits à partir du classeur des inscrits.
        Utilitaire demandé  par Louis Eggermont.
        Version strictement alpha car vérifications sommaires
        de la validité des données (licences et tables)
        """
        bdp = self.barreDeProgression('Vérification des données...\t', 7)
        bdp.start()
        bdp.setValue(1)
        doc = self.doc
        # valider une éventuelle saisie en cours
        self.executeDispatch(".uno:Deselect")
        # récupérer les données
        # (un peu de chipo car pyuno convertit les doubles en floats)
        # ajouté 'tuple(...)' pour compatibilité avec LibO (Python 3.3)
        inscrits = doc.DatabaseRanges.getByName("ImportINSCRITS").ReferredCells.getDataArray()
        inscrits = tuple(map(lambda x: [str(int(y)) if isinstance(y,float) else y for y in x], inscrits))
        # TESTS DE VALIDITÉ
        # prépapation (transformation en dictionnaire)
        dzip = {x[0]:x[1:] for x in zip(*inscrits)}
        # données manquantes ?
        bdp.setValue(2)
        if '' in dzip['Table'] + dzip['Licence']:
            self.msgbox('Des données sont manquantes, merci de le corriger.',
                        'Erreur','warn')
            bdp.end()
            return
        # doublons licences ?
        bdp.setValue(3)
        licences = dzip['Licence'] + dzip['Licence '] 
        for lic in licences:
            if lic != '' and licences.count(lic) >1: # la 2e colonne licence peut être vide
                self.msgbox('Des licences sont en double, merci de corriger.',
                            'Erreur','warn')
                bdp.end()
                return
        # licences mal formées ?
        bdp.setValue(4)
        for lic in licences:
            if lic != '' and len(lic) != 7: # la 2e colonne licence peut être vide
                self.msgbox('Certaines licences sont erronées, merci de corriger.',
                            'Erreur','warn')
                bdp.end()
                return
        # tables mal formées ou en double ?
        bdp.setValue(5)
        if self.infos['tablesparalleles']:
            tables = zip(dzip['F'],dzip['Table'])
        else:
            tables = dzip['Table']
        for table in tables:
            if tables.count(table) > 1:
                self.msgbox('Des tables sont en double, merci de corriger.',
                            'Erreur','warn')
                bdp.end()
                return
        for table in dzip['Table']:
            try:
                int(table)
            except ValueError:
                self.msgbox('Certains numéros de tables sont erronés, merci de corriger.',
                            'Erreur','warn')
                bdp.end()
                return
        # self.msgbox('ok')
        # tout est ok -> on injecte
        bdp.setTextValue('Modification de la base...\t', 6)
        sql = ''
        for inscrit in inscrits[1:]:
            fixe = inscrit[2] and True or False
            d = {'id':inscrit[0], 'lic1':inscrit[1], 'lic2':inscrit[9] or 'Null',
                 'table':inscrit[3], 'fixe':fixe, 'partie':self.infos['derpartie'] or 1}
            sql += ('UPDATE "T_Parties" SET "Licence" = {lic1}, "Licence2" = {lic2}, '
                    '"TableEnCours" = {table}, "TableProvisoire" = {table}, "T{partie}" = {table}, "Fixe" = {fixe} '
                    'WHERE "ID_Partie" = {id};\n').format(**d)
        # self.callXray(sql)
        self.execute(sql)
        bdp.end()
        doc.DatabaseRanges.getByName("ImportINSCRITS").refresh()
        doc.store()
        self.msgbox('\nPARTIE %d\n\n'
                    'Modifications enregistrées.' % (self.infos['derpartie'] or 1))

    def exportDuplitop(self, source):
        '''Exporte la liste des joueurs au format DUPLITOP'''
        bdp = self.barreDeProgression('Création du fichier Duplitop...\t', 8)
        bdp.start()
        try:
            bdp.setValue(1)
            derpartie = self.infos['derpartie']
            nom = self.infos['nom']
            encodage = "ANSI"  # "UTF-8"
            form = source.Model.Parent
            form.reload() # garantit la mise à jour
            totJoueurs = form.getShort(form.findColumn("Licences"))
            totTables = form.getShort(form.findColumn("Tables"))
            # vérifie que les tables ont été attribuées...
            bdp.setValue(2)
            if not totJoueurs:
                self.msgbox("Aucun inscrit !", "Erreur", 'warn')
                return
            elif not derpartie:
                if totTables < totJoueurs:
                    self.msgbox ("Les tables n'ont pas été ou sont incomplètement attribuées.\t", "Erreur", 'warn')
                    return
                derpartie = 1
            bdp.setValue(3)
            provtable = "provTXT"
            if utils.connexion.Tables.hasByName(provtable): utils.connexion.Tables.dropByName(provtable)
            reptournoi = self.convertFromUrl('{tooournoiPath}{nom}'.format(**self.infos))
            suffixe = "_DUPLITOP_partie_%d.txt" % derpartie
            fichier = self.convertFromUrl('%s/%s%s' % (reptournoi, nom, suffixe))
            bdp.setValue(4)
            try:
                os.remove(fichier)
            except OSError:  # le fichier n'existe pas
                try:
                    os.mkdir(reptournoi)
                except OSError:  # le répertoire existe déjà
                    pass
            # instruction SQL qui créée une table texte, affecte le fichier duplitop comme source
            # insère les données tables + joueurs (avec symbole fixe au besoin)
            # et finalement supprimer la table texte
            bdp.setValue(5)
            SQL = ( 'CREATE TEXT TABLE "{0}" ("T" VARCHAR(10), "NOM" VARCHAR(51));\n'
                    'SET TABLE "{0}" SOURCE "{1};encoding=latin-1";\n'
                    'INSERT INTO "{0}" (SELECT CASE WHEN "TablesParalleles"\n'
                    '                          THEN CASE WHEN "Fixe"\n'
                    '                               THEN "SymboleFixe" || "TableEnCours" \n'
                    '                               ELSE \'\'||"TableEnCours" END\n'
                    '                          ELSE \'\'||"TableEnCours" END AS "Table", \n'
                    '      "T_Joueurs"."Nom" || \' \' || "T_Joueurs"."Prénom" AS "Joueur" \n'
                    'FROM "T_Parties", "T_Joueurs", "T_Infos" \n'
                    'WHERE "T_Parties"."Licence" = "T_Joueurs"."Licence" \n'
                    '      AND "T_Parties"."Infos" = "T_Infos"."ID_Infos" \n'
                    '      AND "T_Parties"."ID_Partie" > 0 \n'
                    'ORDER BY CASE WHEN LEFT("Table", LENGTH("SymboleFixe")) = "SymboleFixe" \n'
                    '      AND "TablesParalleles" THEN 1 ELSE 0 END, "TableEnCours");\n'
                    'DROP TABLE "{0}"').format(provtable, fichier)
            self.executeQuery(SQL)
            bdp.setValue(6)
            if utils.databasedocument.isModified(): utils.databasedocument.store()
        finally:
            bdp.end()
        self.msgbox('\nLe fichier :\n\n< %s%s >'
                    '\n\nest disponible à l\'adresse :\n\n'
                    '< %s >\t\n\n' % (nom, suffixe, reptournoi), "", 'info' )

    def exportInscritsCSV(self, source):
        """Exporte le fichier d'inscrits au format csv"""
        # from com.sun.star.ui.dialogs.TemplateDescription import FILESAVE_SIMPLE
        # O.filePicker(filters=(("Fichiers CSV (*.csv)", "*.csv"),), mode=FILESAVE_SIMPLE)
        exportrange = self.doc.DatabaseRanges.getByName('ImportINSCRITS').ReferredCells
        URL = self.convertFromUrl(self.doc.URL)[:-3] + 'csv'
        self.dataarray2csv(URL, exportrange.DataArray)


