# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
from com.sun.star.sdb import RowSetVetoException
import utils

class Joueurs(utils.Tournoi):
    arrayjoueurs = []

    def __init__(self):
        super(Joueurs, self).__init__()

    def verifDoublonClassement(self, source):
        '''Vérifie que le classement n'est pas déjà attribué'''
        form = self.doc.DrawPage.Forms.getByName('MainForm')
        classement = source.CurrentValue
        # cas particuliers
        if not form.RowCount or not source.Text:
            return True
        elif source.Text == "0":
            self.msgbox("Zéro non autorisé pour ce champ.\t", "Erreur",'warn')
            form.reset()
            return False
        elif classement == source.BoundField.Value:
            return True
        # cas normaux
        clone = form.createClone()
        clone.setPropertyValues(('ApplyFilter','Filter'),(True,'Classement = %d' % classement))
        clone.execute()
        if clone.RowCount:
            self.msgbox("Un joueur est déjà classé au rang %d.\n"
                        "Il serait préférable de choisir une autre valeur.\t" % classement, "Erreur", 'warn')
        clone.dispose()
        return True

    def verifDoublonJoueur(self, source):
        '''Vérifie que le joueur n'est pas déjà dans la base'''
        form = self.doc.DrawPage.Forms.getByName('MainForm')
        licence = source.Text.strip()
        # cas particuliers
        if licence == source.BoundField.String:   # si pas de modif
            return True
        elif not licence:  # si aucune saisie ou si effacement
            source.BoundField.updateNull()
            form.reset()
            return True
        elif len(licence) < 7:   # si longueur licence non valide
            self.msgbox("Ce numéro de licence n'est pas valable.\t", "Avertissement", 'warn')
        # cas normaux
        else:
            joueur = self.nomDuJoueur(licence)
            if not joueur:  # si le joueur n'existe pas déjà, c'est OK
                return True
            else:
                self.msgbox("%s est déjà dans la base.\t" % joueur, "Avertissement", 'warn')
        if form.IsNew:
            form.refreshRow()
            form.moveToInsertRow()
        else:
            form.cancelRowUpdates()
        form.reset()

    def filtreGeneral(self, source):
        '''Active un filtre rapide et global'''
        texte = source.Text
        texte.replace("'","''")   # double les apostrophes pour le SQL
        filtre = ("Classement LIKE '%{texte}%'"
                  " OR Nom LIKE '%{texte}%'"
                  " OR \"Prénom\" LIKE '%{texte}%'"
                  " OR Licence LIKE '%{texte}%'"
                  " OR \"Série\" LIKE '%{texte}%'"
                  " OR Club LIKE '%{texte}%'").format(texte=texte)
        form = self.doc.DrawPage.Forms.getByName('MainForm')
        form.setPropertyValues(('ApplyFilter','Filter'),(True,filtre))
        form.reload()
        # supprimer le filtre si résultat du reload précédent est null
        if not form.RowCount:
            form.ApplyFilter = False
            form.reload()

    def ferme_F_Joueurs(self, source):
        '''Gestion fermeture'''
        docJoueurs = utils.databasedocument.FormDocuments.getByName("F_Joueurs")
        form = source.Model.Parent
        controleLic = form.TableControl.getByName('colLicence')
        # valide changements si nouvel enregistrement...
        try:
            if form.IsNew:
                # si le nouvel enregistrement n'est pas vierge (sinon boucle sur licence vide)
                if form.IsModified: 
                    if len(controleLic.Text.strip()) < 7:
                        self.msgbox("Le numéro de licence n'est pas valable.\t", "Avertissement", 'warn')
                        return
                    else:
                        form.insertRow()  # insère l'enregistrement
            else:
                form.updateRow()   # update l'enregistrement
            self.executeDispatch(".uno:CloseDoc", frame=form.Parent.Parent.CurrentController.Frame)
        except RowSetVetoException:
            pass   #a priori le message qui précède, interne à OOO, est suffisant

    def ferme_F_Joueurs_sans_sauver(self, source):
        docJoueurs = utils.databasedocument.FormDocuments.getByName("F_Joueurs")
        form = source.Model.Parent
        if form.IsNew:
            form.reset()
        else: form.cancelRowUpdates()
        self.executeDispatch(".uno:CloseDoc", frame=form.Parent.Parent.CurrentController.Frame)

    def choixFichierJoueurs(self, source):
        '''Associé à l'événement 'texte modifié' du controle FileControl,
        vérifie la validité du fichier et récupère les en-têtes de colonnes'''
        nbcol = 9
        utils.colonnesPos = [-1]*nbcol
        dlg = source.Context
        # charge les données dans un classeur Calc non visible
        try:
            # url locale
            url = self.convertToUrl(dlg.getControl("FileControl").Text)
        except:
            # url distante
            url = dlg.getControl("FileControl").Text
        doc = self.importViaCalc(url, Hidden=True, coldef='1/2/2/2/3/2/4/2/5/2/6/2/7/2/8/2/9/2') # 2 = texte
        if not doc:    # url inexistante -> on ignore
            pass
        else:
            # récupère le choix 'Duplitop'
            duplitop = dlg.getControl("cocherDuplitop").State
            # récupère les données
            del self.arrayjoueurs[:]
            try:
                curseur = doc.CurrentController.ActiveSheet.createCursor()
            except AttributeError:
                self.msgbox("Ce type de fichier n'est pas pris en charge.\t", "Erreur", 'warn')
                return
            else:
                curseur.gotoStartOfUsedArea(False)
                curseur.gotoEndOfUsedArea(True)
                self.arrayjoueurs.append(curseur.getDataArray())
            finally:
                doc.close(True)
            # if len(self.arrayjoueurs[0]) > 5000:
                # self.msgbox("Le fichier contient un grand nombre de joueurs.\n"
                            # "L'importation pourrait être longue, soyez patient...", "Avertissement", 'warn')
            # vérifie le nombre de colonnes
            if len(self.arrayjoueurs[0][0]) < nbcol:
                self.msgbox("Attention : certaines colonnes manquent dans le fichier source.\n"
                            "Toutes les colonnes sont obligatoires.", "Avertissement", 'warn')
        # active les listbox et les alimente avec en-têtes de colonne
        for n in range(nbcol):
            lbox = dlg.getControl("boxEnTete%d" % (n+1))
            lbox.removeItems(0, lbox.ItemCount )    # efface si une première sélection à remplacer
            if doc:
                lbox.addItems(self.arrayjoueurs[0][1],0)
                lbox.DropDownLineCount = lbox.ItemCount #cosmétique
                if duplitop:
                    lbox.selectItemPos(n, True)
                # else: lbox.setEnable(True)

    def importJoueurs(self, bdp, defaultURL=''):
        '''Importation du fichier et nettoyage des erreurs éventuelles.'''
        from datetime import datetime

        def verifiejoueur(i, row):
            # i = 1#joueurs.index(row) + 1
            row[nNom] = row[nNom].strip().replace("'","''")
            if not row[nPre].strip():
                row[nPre] = "<?>"
            else:
                row[nPre] = row[nPre].strip().replace("'","''")
            if not row[nLic].isdigit() or len(row[nLic]) != 7:
                if i > 1: self.logjoueurs += "%d : %s %s (licence invalide)\n" % (i, row[nNom], row[nPre])
                return False
            elif not row[nNom]:
                if i > 1: self.logjoueurs += "%d : nom du joueur manquant\n" % (i)
                return False
            if not row[nCla].isdigit(): row[nCla] = "NULL"
            if not row[nCat].isalpha(): row[nCat] = "S"
            if not row[nSer].isalnum(): row[nSer] = "?"
            if not row[nClu].isalnum(): row[nClu] = "???"
            if not row[nFed].isalnum(): row[nFed] = "??"
            if not row[nNat].isalnum(): row[nNat] = "??"
            if row[nSer].upper() == 'NC': row[nSer] = "7"
            return True

        # création du dialogue
        dlg = self.createTooournoiDialog('dialogue5')
        if defaultURL: dlg.getControl("FileControl").setText(defaultURL)
        bdp.end()
        while 1:
            if not dlg.execute():
                return
            elif -1 in utils.colonnesPos:
                self.msgbox("Tous les champs doivent être associés.", "Erreur", 'warn')
            else:
                break
        nNom, nPre, nLic, nCla, nSer, nClu, nCat, nFed, nNat = utils.colonnesPos
        # récupère le mode d'importation
        ecraser = dlg.getControl("optEcraser").State
        # lancer la récupération
        bdp = self.barreDeProgression('Mise à jour de la liste des joueurs...\t', 7)
        bdp.start()
        try:
            bdp.setValue(1)
            # transformation en listes de chaînes
            # (un peu de chipo car pyuno convertit les doubles en floats
            bdp.setValue(2)
            joueurs = map(lambda x: list(x), self.arrayjoueurs[0])
            # on élimine et on logue les lignes malformées
            bdp.setValue(3)
            self.logjoueurs = ''
            joueurs = [j for n,j in enumerate(joueurs,1) if verifiejoueur(n,j)]
            # on reclasse selon l'ordre récupéré des colonnes
            bdp.setValue(4)
            # ajouté 'tuple()' pour compatibilité avec python3 (LibO)
            joueurs = tuple(map(lambda row: [x for y, x in sorted(zip(utils.colonnesPos, row))], joueurs))
            # on construit la requête
            bdp.setValue(5)
            insert = ('INSERT INTO "T_Joueurs" ("Nom", "Prénom", "Licence", '
                      '"Classement", "Série", "Club", "Cat", "Fédération", "Nationalité") '
                      'VALUES (UPPER(\'%s\'), \'%s\', %s, %s, \'%s\', \'%s\', \'%s\', \'%s\', \'%s\');')
            if ecraser:
                self.execute('DELETE FROM "T_Joueurs";')
                inserts = (insert % tuple(x) for x in joueurs)
                sql = '\n'.join(inserts)
                # on l'exécute
                bdp.setValue(6)
                self.execute(sql)
            else:
                from com.sun.star.sdbc import SQLException
                bdp.end()
                bdp = self.barreDeProgression('Mise à jour de la liste des joueurs...\t', len(joueurs))
                bdp.start()
                # on l'exécute joueur par joueur (pas de INSERT INTO... IF NOT EXISTS hélas )
                for n, joueur in enumerate(joueurs,1):
                    bdp.setValue(n)
                    try:
                        self.execute(insert % tuple(joueur))
                    except SQLException:
                        pass
            self.forms_reload()
        finally:
            bdp.end()
        # ' fin de la procédure et log éventuel
        if self.logjoueurs:
            dlgLog = self.createTooournoiDialog('dialogue6')
            texteLog = ("Importation terminée.\n"
                        "Les lignes suivantes n'ont pas été importées :\n\n%s\n"
                        % self.logjoueurs)
            dlgLog.getControl("bTexte").setText(texteLog)
            dlgLog.execute()
            dlgLog.dispose()
        # enregistrer la date de mise à jour
        cles = self.clesOptions()
        cles.setPropertyValue("PlayersUpdateDate", datetime.strftime(datetime.now(), '%Y%m%d'))
        cles.commitChanges()

