# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import string
import utils
import traceback


class Outils(utils.Tournoi):
    SELECTCOLOR = 16750950  # orange foncé
    UNSELECTCOLOR = 16764057    # orange clair
    CARINTERDITS = r'\/:*?"<>|'

    def __init__(self):
        super(Outils, self).__init__()
        self.infos['suffixe_resultats'] = "Resultats"
        self.infos['suffixe_annexes'] = "Annexes"

    def prepareImportTournoi(self):
        '''Lance le dialogue d'importation'''
        # mémorise la fenêtre l'affichage du dialogue final dans la route importerTournoi
        utils.frametOOournoi = self.getCurrentFrame()
        # ouvrir le fichier contenant les données
        files = self.filePicker()
        if not files: return
        doc = self.importViaCalc(files[0])
        if not doc:
            self.msgbox("Ce fichier n'est pas valide.","Erreur",'warn')
            return
        # Récupérer les paramètres d'importation
        utils.dlg9 = self.createTooournoiDialog('dialogue9')
        utils.dlg9.Peer.invalidate(8)
        utils.dlg9.setVisible(True)

    def importerTournoi(self, source):
        '''Récupère un fichier csv ou classeur et,
        en fonction des paramètres d'importation fourni,
        importe les données'''
        doc = self.doc
        dlg = utils.dlg9
        if not source.Model.Name == "btnOK":
            doc.close(True)
            dlg.dispose()
            return

        nouv = dlg.getControl('optNouveau').State
        topsparties = int(dlg.getControl('numTopsJoueurs').Value)
        premjoueur = int(dlg.getControl('numPremJoueur').Value)
        lic1 = dlg.getControl('comboLic1').Text.strip()
        lic2 = dlg.getControl('comboLic2').Text.strip()
        simultane = dlg.getControl('simultane').State
        topsjoueurs = dlg.getControl('txtTopsJoueurs').Text.replace(' ','').replace(';',',').upper().rstrip(',')
        if not (lic1 and topsparties and premjoueur and topsjoueurs):
            self.msgbox("Paramètres d'importations incomplets\t","Erreur",'warn')
            return
        dlg.dispose()
        
        # traitement des données
        f = doc.CurrentController.ActiveSheet
        c = f.createCursor()
        c.gotoEndOfUsedArea(True)
        # on est obligé de récupérer des 'string' plutôt que des 'value', certaines données
        # pouvant exister comme du texte (j'ai eu le cas). Du coup, si certaines données sont dans
        # un format exotique, le texte pourrait ne pas être interprétable.
        # La ligne qui suit à donc pour but de supprimer tout format de nombre autre que Standard.
        c.NumberFormat = doc.NumberFormats.getStandardIndex(doc.DocumentProperties.Language)
        # ne pas oublier le '+1', puisqu'on utilisera les noms de cellules et pas les index
        maxRows = c.RangeAddress.EndRow+1
        prefix_ins = 'INSERT INTO "T_Parties" ("Licence","Licence2"'
        prefix_upd = 'UPDATE "T_Parties" SET'
        topsjoueurs = topsjoueurs.split(',')
        nbParties = len(topsjoueurs)
        if nbParties > self.infos['maxpartie']:
            self.modifierNombreParties(nbParties)
            self.executeQuery('UPDATE "T_Infos" SET "PartiesMax" = %d;' % nbParties)

        if nouv: self.resetTournoi(nbParties)
        for partie, col in enumerate(topsjoueurs, 1):
            if col.isalpha():   # permet de sauter des parties (si on ne veut importer que la 2e par exemple)
                prefix_ins += ',"P%d"' % partie
                prefix_upd += ' "P%d"={},' % partie
                top = int(f.getCellRangeByName('%s%d' % (col,topsparties)).String)
                # une première ligne est créée via resetTournoi, on peut donc déjà entrer les tops :
                self.executeQuery('UPDATE "T_Parties" SET "P%s" = %d WHERE "ID_Partie"=0' % (partie,top))
        prefix_ins += ") VALUES (%d"
        prefix_upd = prefix_upd[:-1] # suppression dernière virgule
        sSQLinsert, sSQLupdate = "", ""
        deflic = (l for l in range(9999999,0,-1))
        for lig in range(premjoueur, maxRows+1):
            # si case vide dans colonne licence, on quitte la boucle
            # (la plage utilisée peut bien sûr être plus grande que
            # la stricte plage de résultats)
            lic = f.getCellRangeByName('%s%d' % (lic1,lig)).String
            if lic == "": break
            try:
                lic = int(lic[:7])
            except ValueError:
                lic = next(deflic)
            if self.estInscrit(lic): # cas des classements cumulés sur plusieurs tournois
                points = []
                for col in topsjoueurs:
                    if col.isalpha():
                        points.append(f.getCellRangeByName("%s%d" % (col,lig)).String or '0')
                sSQLupdate += prefix_upd.format(*points)
                sSQLupdate += ' WHERE "Licence" = {}\n'.format(lic)
            else:
                # insertion licence 1
                sSQLinsert += prefix_ins % lic
                # insertion licence 2
                if lic2:
                    lic = f.getCellRangeByName('%s%d' % (lic2,lig)).String
                    try:
                        sSQLinsert += ',%s' % int(lic[:7])
                    except ValueError:
                        sSQLinsert += ',%s' % next(deflic)
                else:
                    sSQLinsert += ",null"
                # insertion points parties
                for col in topsjoueurs:
                    if col.isalpha():   # permet de sauter des parties (si on ne veut importer que la 2e par exemple)
                        sSQLinsert += ",%s" % (f.getCellRangeByName("%s%d" % (col,lig)).String or '0')
                sSQLinsert += ");\n"
        # self.msgbox(sSQLinsert)
        # self.msgbox(sSQLupdate)
        if len(sSQLinsert) > len(prefix_ins):
            self.executeQuery( sSQLinsert )
        if len(sSQLupdate) > len(prefix_upd):
            self.executeQuery( sSQLupdate )
        self.executeQuery(('UPDATE "T_Parties" SET "TableEnCours" = (SELECT MAX("TableEnCours") FROM "T_Parties")+ "ID_Partie" '
                          'WHERE "ID_Partie" > 0 AND "TableEnCours" IS NULL;\n'))
        # indispensable de n'avoir aucune table manquante
        for partie, col in enumerate(topsjoueurs, 1):
            self.executeQuery(('UPDATE "T_Parties" SET "T{0}" = "TableEnCours" '
                               'WHERE "ID_Partie" > 0 AND "T{0}" IS NULL;\n').format(partie))
        if nouv:
            sSQL = ('UPDATE "T_Parties" SET "Infos" = 0;\n'
                    'UPDATE "T_Infos" SET "Paires" = %s;\n'
                    'UPDATE "T_Infos" SET "PartiesMax" = %d;\n'
                    'UPDATE "T_Infos" SET "NomTournoi" = \'%s\''
                     % (bool(lic2), nbParties, self.convertFromUrl(doc.URL.split('/')[-1].rsplit('.',1)[0])) )
            self.executeQuery( sSQL )
        doc.close(True)

        # calcul classement
        from parties import Parties
        P = Parties()
        P.frame = self.frame
        sSQL = ""
        for x in range(1, nbParties+1):
            if not simultane:
                # mettre '0' aux joueurs absents
                sSQL += ('UPDATE "T_Parties" SET "P{0}"=0 WHERE "P{0}" IS NULL AND "ID_Partie" > 0;\n').format(x)
            # simuler l'initiation de la partie
            sSQL += 'UPDATE "T_Parties" SET "T{0}"=0 WHERE "ID_Partie" = 0;\n'.format(x)
            self.executeQuery( sSQL )
            # le sql précédent doit avoir été exécuté avant de recalculer le rang
            P.calculeRang(x)
        self.executeQuery(('UPDATE "T_Infos" SET "PartiesJouees"={0} '
                           'WHERE "ID_Infos" = 0 AND "PartiesJouees" < {0}\n').format(x))
        if nouv:
            self.ouvreForm("Menu_Options")
        else:
            self.forms_reload()
            # petit hack pour l'affichage du dialogue à cause du changement de contexte
            self.frame = utils.frametOOournoi
            self.msgbox('Importation réussie')
        self.forms_reload()

    def defragmenter(self):
        '''Défragmentation, si nécessaire'''
        # ouvre barre de progression symbolique
        bdp = self.barreDeProgression('Défragmentation\t', 3)
        bdp.start()
        bdp.setValue(1)
        utils.databasedocument.store()
        bdp.setValue(2)
        self.execute( "CHECKPOINT DEFRAG" )
        # alimente et ferme la barre de progression
        bdp.setValue(3)
        bdp.end()

    def exportODS(self, ouvrir=True):
        '''Export des données dans tableur'''
        from com.sun.star.document.MacroExecMode import ALWAYS_EXECUTE_NO_WARN

        # ouvre barre de progression
        bdp = self.barreDeProgression("Sauvegarde de la base...")
        bdp.start()
        bdp.setValue(30)
        utils.databasedocument.store()
        bdp.setValue(40)
        # on ouvre le classeur modèle
        url = self.getTOOournoiPath()
        nouvURL = self.convertToUrl('{tooournoiPath}{nom}/{nom}_{suffixe_resultats}.ods'.format(**self.infos))
        oDoc = self.docSiOuvert(nouvURL)
        if oDoc:
            self.msgbox("Un fichier de résultats est déjà ouvert !\t", "Avertissement", 'warn')
            bdp.end()
            return
        modele = self.convertToUrl( '{extPath}modele_{suffixe_resultats}.ots'.format(**self.infos))
        args = self.createPropertyValues(MacroExecutionMode=ALWAYS_EXECUTE_NO_WARN,
                                         AsTemplate=True,
                                         Hidden=True)
        oDoc = self.loadComponentFromURL(modele, args=args)
        try:
            bdp.setTextValue("Création du fichier...", 60)
            oSP = oDoc.ScriptProvider
            oScript = oSP.getScript("vnd.sun.star.script:Standard.Start.Demarrage_?language=Basic&location=document")
            oScript.invoke((),(),())
            bdp.setValue(75)
            # on supprime les macros devenues inutiles
            oDoc.BasicLibraries.loadLibrary("Standard")
            library = oDoc.BasicLibraries.getByName("Standard")
            bdp.setValue(90)
            if library.hasByName("Start"): library.removeByName("Start")
            bdp.setValue(99)
            oDoc.storeAsURL(nouvURL, ())
            self.exportCSV(oDoc)
            if ouvrir:
                oDoc.CurrentController.Frame.ContainerWindow.setVisible(True)
            else:
                oDoc.close(True)
        except Exception as e:
            if oDoc:
                oDoc.close(True)
            try:
                self.msgbox(e.Message, "Erreur %d" % e.ErrCode)
            except AttributeError:
                self.msgbox(repr(e), "Erreur")
        finally:
            bdp.end()

    class ThisFormatter(string.Formatter):
        def __init__(self):
            super(Outils.ThisFormatter, self).__init__()
            self.keycount = 0

        def reset(self):
            self.keycount = 0

        def get_value(self, key, args, kwargs):
            if key == '':
                key = self.keycount
                self.keycount += 1
            return super(Outils.ThisFormatter, self).get_value(key, args, kwargs)

        def format_field(self, value, spec):
            if value == None:
                value = ''
            if spec == 't':
                if isinstance(value, (int, float)):
                    return super(Outils.ThisFormatter, self).format_field(value, 'g')
            return super(Outils.ThisFormatter, self).format_field(value, '')

    def exportCSV(self, doc=None):
        fmt = self.ThisFormatter()
        try:
            if not doc:
                doc = self.doc
            filepath = os.path.dirname(self.convertFromUrl(doc.URL))
            feuille = doc.Sheets.getByName("Cumul")
            pa = feuille.PrintAreas[0]
            exportrange = feuille.getCellRangeByPosition(0,9,pa.EndColumn,pa.EndRow)
            filename = "%s_%s.csv" % (feuille.getCellByPosition(1,2).String, 
                            feuille.getCellByPosition(1,4).String)
            #--- adaptation provisoire au format FBS ---
            data = zip(*exportrange.DataArray)
            # insertion parties non jouées
            nbParties = (len(data)-16)/2
            for n in range(nbParties+1, 8):
                row = ("",)*len(data[0])
                data.insert(-8, row)
                data.insert(-8, row)
            # rangs par séries et par catégories
            rangs = {'S':[], 'V':[], 'D':[], 'E':[], 'J':[], 'C':[], 'B':[], 'P':[], 'M':[],
                           '1':[], '2':[], '3':[], '4':[], '5':[], '6':[], '7':[]}
            tops = [x for x in data[7][2:]]
            cat = [x for x in data[4][2:]]
            ser = [str(x)[0] for x in data[3][2:]]
            z_cat = sorted(zip(cat, tops), reverse=True)
            z_ser = sorted(zip(ser, tops), reverse=True)
            for k, v in (z_cat+z_ser):
                rangs[k].append(v)
            # en-tête 1
            csv = []
            data = zip(*data)
            f_info = doc.Sheets.getByName("Infos")
            entete1 = ("{:t};;;;;;{:t} -- S:{:t} - V:{:t} - D:{:t} - E:{:t} - J:{:t} - C:{:t} - B:{:t} - P:{:t} - M:{:t}"
                       ";;;;;;;;;;;;;;;;;;;;;;;;cumul;{:t};{:t};{:t};{:t};{:t};{:t};{:t}")
            nom = feuille.getCellByPosition(1,2).String
            nbjoueurs = feuille.getCellByPosition(1,5).String
            categories = tuple(zip(*f_info.getCellRangeByPosition(10,6,10,14).DataArray)[0])
            series = tuple(zip(*f_info.getCellRangeByPosition(7,6,7,12).DataArray)[0])
            cumulseries = tuple(sum(series[0:e]) for e, v in enumerate(series, 1))
            entete1 = fmt.format(entete1, nom, nbjoueurs, *categories+cumulseries)
            csv.append(entete1)
            # en-tête 2
            fmt.reset()
            entete2 = "{:t};;;;;;{} -- {};;;Cumul;Place;Place;P1;;P2;;P3;;P4;;P5;;P6;;P7;;;%;Écart;;par série;{:t};{:t};{:t};{:t};{:t};{:t};{:t}"
            date = f_info.getCellByPosition(4,12).String
            stats = feuille.getCellByPosition(1,6).String
            stats = stats.replace("  |  ", "-").replace(" : ", ":")
            entete2 = fmt.format(entete2, date, nbjoueurs, stats, *series)
            csv.append(entete2)
            # en-tête 3
            fmt.reset()
            entete3 = "Place;Table ;Licence;NOM Prénom;Cat.;Série;Club;Nat;;{:t};Série;Cat;{:t};Pl.;{:t};Pl.;{:t};Pl.;{:t};Pl.;{:t};Pl.;{:t};Pl.;{:t};Pl.;;;;Pl;;%S1;%S2;%S3;%S4;%PP5;%PP6;%PP7"
            tops = data[1][7], data[1][8], data[1][10], data[1][12], data[1][14], data[1][16], data[1][18], data[1][20]
            entete3 = fmt.format(entete3, *tops)
            csv.append(entete3)
            #restant
            TOP = data[1][7]
            modele = ("{pl:t};{table:t};{lic:t};{nom:t};{cat:t};{ser:t};{club:t};{nat:t};;{top:t};{pls:t};{plc:t};{p1:t};{plp1:t};{p2:t};{plp2:t};{p3:t};{plp3:t};{p4:t};{plp4:t};"
                      "{p5:t};{plp5:t};{p6:t};{plp6:t};{p7:t};{plp7:t};;{pct:t};{ecart:t};{pl:t};;{S1:t};{S2:t};{S3:t};{S4:t};{S5:t};{S6:t};{S7:t}")
            for j in data[2:]:
                data = dict(pl=j[1], table='', lic=j[0], nom=j[2], ser=j[3], cat=j[4], club=j[5], nat=j[6], top=j[7], pls=rangs[str(j[3])[0]].index(j[7])+1,
                            plc=rangs[j[4]].index(j[7])+1, p1=j[8], plp1=j[9], p2=j[10], plp2=j[11], p3=j[12], plp3=j[13], p4=j[14], plp4=j[15], p5=j[16],
                            plp5=j[17], p6=j[18], plp6=j[19], p7=j[20], plp7=j[21], pct=j[-8]*100, ecart=j[7]-TOP, S1=j[-7]*100, S2=j[-6]*100, S3=j[-5]*100,
                            S4=j[-4]*100, S5=j[-3]*100, S6=j[-2]*100, S7=j[-1])
                ligne = fmt.format(modele, **data)
                csv.append(ligne)
            # export
            import codecs
            path = os.path.join(filepath,filename)
            with codecs.open(path, 'wb', encoding='latin1') as f:
                f.write('\n'.join(csv))
            return True
        except Exception as err:
            print("outils.Outils.exportCSV\n--------")
            traceback.print_exc()
            self.msgbox("outils.Outils.exportCSV\n--------\nErreur: {0}".format(traceback.format_exc()))
            return False

    def mailtofede(self):
        '''Prépare l'envoi des résulats par mail'''
        fichier_ods = self.convertFromUrl('{tooournoiPath}{nom}/{nom}_{suffixe_resultats}.ods'.format(**self.infos))
        fichier_csv = self.convertFromUrl('{tooournoiPath}{nom}/{nom}_{suffixe_resultats}.csv'.format(**self.infos))
        fichiers = (fichier_ods, fichier_csv)
        mailto = self.clesOptions().SendMailTo
        mailto.replace(' ','')
        mailto.replace(',',';')
        mailto = tuple(mailto.split(";"))
        if not self.infos['derpartie']:
            self.msgbox("Aucune partie n'a encore été initiée.\t", "Erreur", 'warn')
            return
        elif not os.access(fichier_ods, os.F_OK) or not os.access(fichier_csv, os.F_OK):
            self.exportODS(False)
        # SimpleSystemMail n'est plus supporté par AOOO, tandis que
        # SystemMailProvider n'est pas supporté par LibO
        try:
            systemmail = self.createUnoService("com.sun.star.system.SimpleSystemMail")
            client = systemmail.querySimpleMailClient()
            message = client.createSimpleMailMessage()
            message.Attachement = fichiers
            message.Recipient = mailto[0]
            message.CcRecipient = mailto[1:]
            message.Subject = "[tOOournoi] Envoi automatique des résultats : %s" % self.infos['nom']
            client.sendSimpleMailMessage(message, 0)
        except AttributeError:
            systemmail = self.createUnoService("com.sun.star.system.SystemMailProvider")
            client = systemmail.queryMailClient()
            message = client.createMailMessage()
            message.Attachement = fichiers
            message.Recipient = mailto[0]
            message.CcRecipient = mailto[1:]
            message.Subject = "[tOOournoi] Envoi automatique des résultats : %s" % self.infos['nom']
            message.Body = ("Ci-joint les résultats du tournoi dont objet,\n"
                           "y compris également les annexes de clôture.\n\n"
                           "Bien cordialement.")
            # message d'erreur si envoi avorté par utilisteur -> afficher message proprement
            from com.sun.star.uno import Exception
            try:
                client.sendMailMessage(message, 0)
            except Exception as e:
                print("outils.Outils.mailtofede\n--------")
                traceback.print_exc()
                self.msgbox("outils.Outils.mailtofede\n--------\nErreur: {0}".format(traceback.format_exc()))
            
    def charger_archives(self, source):
        '''Permet de charger une partie archivée'''
        form = source.Model.Parent
        if not form.RowCount:
            self.msgbox("Aucune archive disponible.", "Erreur", 'warn')
            return
        else:
            nArchive = form.getString(1)
            if nArchive[0] == '>':
                self.msgbox("Ce tournoi est déjà chargé !", "Avertissement", 'warn')
                return
        bdp = self.barreDeProgression("Chargement...\t", 4)
        bdp.start()
        bdp.setValue(1)
        self.cree_archives()
        bdp.setValue(2)
        SQL = ('DELETE FROM "T_Infos";\n'
               'INSERT INTO "T_Infos" SELECT * FROM "T_Infos_%s"') % nArchive
        self.execute(SQL)
        # la propriété <self.infos['maxpartie']> doit à ce stade être rechargée !
        self.modifierNombreParties(self.getTournoiInfo('PartiesMax', 'Int'))
        SQL = ('DELETE FROM "T_Parties";\n'
               'INSERT INTO "T_Parties" SELECT * FROM "T_Parties_%s"') % nArchive
        self.execute(SQL)
        SQL = ('DELETE FROM "T_Acteurs";\n'
               'INSERT INTO "T_Acteurs" SELECT * FROM "T_Acteurs_%s"') % nArchive
        self.execute(SQL)
        # 'actualiserTables
        bdp.setValue(3)
        bdp.end()
        self.msgbox("Les données de :\n\n\t>> %s <<\t"
                    "\n\nont été correctement chargées." % nArchive, "Chargement réussi")

    def supprimer_archives(self, source):
        '''Permet de charger une partie archivée'''
        form = source.Model.Parent
        if not form.RowCount:
            self.msgbox("Aucune archive disponible.", "Erreur", 'warn')
            return
        else:
            nArchive = form.getString(1)
            if nArchive[0] != '>':
                if self.msgbox('Êtes-vous certain de vouloir supprimer définitivement\t\n'
                               'les données archivées de "%s" ?' % nArchive, 
                               'Confirmation de suppression', "warn",
                               boutons=('yesno','defno')) == 3:
                    return
                # ' on nettoie fichiers devenus obsolètes
                aURL = utils.databasedocument.URL.split("/")
                aURL[-1] = nArchive
                rep = self.convertFromUrl('/'.join(aURL)+'/')
                # ' nettoie les fichiers de classement, qui seront obsolètes, et le sous-répertoire correspondant si vide
                if os.path.isdir(rep):
                    try:
                        if os.path.isfile("%s_Cumul.ods" % (rep+nArchive)): os.remove("%s_Cumul.ods" % (rep+nArchive))
                        if os.path.isfile("%s_Partie.ods" % (rep+nArchive)): os.remove("%s_Partie.ods" % (rep+nArchive))
                        if os.path.isfile("%s_Inscrits.ods" % (rep+nArchive)): os.remove("%s_Inscrits.ods" % (rep+nArchive))
                        if not os.listdir(rep): os.rmdir(rep)
                    except OSError as e:
                        if e.errno == 13:
                            self.msgbox("Impossible d'effacer ce tournoi.\n"
                                        "Vérifiez que tous les fichiers ont bien été fermés.\t", "Erreur",'warn')
                            return
                        else: raise
            else:
                nArchive = nArchive[1:]
            SQL = ('DROP TABLE "T_Parties_%(archive)s" IF EXISTS;\n'
                   'DROP TABLE "T_Infos_%(archive)s" IF EXISTS ;\n'
                   'DROP TABLE "T_Acteurs_%(archive)s" IF EXISTS ;\n'
                   'DROP TABLE "T_duplitableurs_%(archive)s" IF EXISTS ;\n'
                   'DROP TABLE "T_dupliparties_%(archive)s" IF EXISTS ;') % {'archive': nArchive}
            self.execute(SQL)
            utils.connexion.Tables.refresh()

    def toggleOptionTablesFixes(self, source):
        '''Afficher/masquer les options pertinentes suivant le contexte.'''
        if self.hasUnoInterface(source, 'com.sun.star.form.XForm'):
            form = source
            etat = self.infos['tablesaumerite']
        else:
            form = source.Model.Parent
            etat = source.State
        for n in range(5):
            form.getByName('tfixes%d' % n).Enabled = etat

    def changeOnglet(self, nom):
        form = self.doc.DrawPage.Forms.getByName("Standard_Menu_Principal")
        sections = self.doc.TextSections
        for section in self.iterUno(sections):
            if section.Name == nom:
                section.IsVisible = True
                form.getByName(section.Name).BackgroundColor = self.SELECTCOLOR
            else:
                section.IsVisible = False
                form.getByName(section.Name).BackgroundColor = self.UNSELECTCOLOR

    def resetteur(self):
        '''Démarrer un nouveau tournoi (séquence d'avertissement)'''
        if self.infos['nom'] != self.starttext:
            url = self.convertFromUrl("%(tooournoiPath)s%(nom)s/%(nom)s_Resultats.ods" % self.infos)
            if not os.path.isfile(url):
                choix = self.msgbox("\nIl est vivement conseillé d'EXPORTER les données avant de poursuivre.\t\n"
                                    "Souhaitez vous le faire maintenant ?\n\n",
                                    "Avertissement", 'warn', boutons=('yesnocancel',))
                if not choix:
                    return
                elif choix == 2:
                    self.exportODS(False)
            elif self.msgbox("\nÊtes-vous certain de vouloir réinitialiser le programme ?\n\n"
                             "(Les données actuelles pourront être rechargées via le menu Outils.)\t\n\n",
                             "Avertissement", 'warn', boutons=('yesno','defno')) == 3:
                return
        # ' le RESET proprement dit
        self.resetTournoi(1)
        # ' rafraîchit la couche formulaire
        self.forms_reload()
        # ' ouvre le formulaire de paramètres
        self.ouvreForm("Menu_Options")

    def verifiePartiesJouees(self, source):
        '''S'assurer que le nombre de parties à jouer ne soit pas
        inférieur au nombre de parties déjà initiées'''
        parties = int(source.CurrentValue)
        if parties < self.infos['derpartie']:
            self.msgbox("\nLe nombre de parties ne peut être inférieur\t"
                        "\nau nombre de parties déjà initiées.\n\n",
                        "Erreur", 'warn')
            source.reset()
            return False
        else:
            self.modifierNombreParties(parties)   #' voir routine dans utils
            return True

    def refuseChaineVide(self, source):
        '''Force à nommer le tournoi'''
        txt = source.Text = source.Text.strip()
        # ev.source.text = txt
        if not txt:
            self.msgbox("Il est obligatoire de nommer la compétition.\t", "Avertissement", 'warn')
            self.doc.CurrentController.getControl(source).setFocus()
            source.reset()
            return False
        # ' un copier/coller pourrait contourner la routine 'car_interdits'
        elif set(txt) & set(self.CARINTERDITS):
            txt = ''.join(x for x in txt if x not in self.CARINTERDITS)
            source.Text = txt
            self.msgbox("Les caractères suivants sont interdits et ont été supprimés :\n\n%s" % ' '.join(self.CARINTERDITS))
        return True

    def car_interdits(self, source, keychar):
        '''Supprime les caractères interdits en cours de frappe'''
        if keychar in self.CARINTERDITS:
            sel = source.getSelection()
            sel.Min, sel.Max = sel.Min-1, sel.Min-1
            self.msgbox("Les caractères suivants sont interdits :\n\n%s" % ' '.join(self.CARINTERDITS))
            source.setText(source.Text.replace(keychar,''))
            source.setSelection(sel)

    def oblige_symbole_fixe(self, source):
        txt = source.Text
        if not txt.strip():
            self.msgbox("Ce champ ne peut rester vide.\t", "Avertissement", 'warn')
            self.doc.CurrentController.getControl(source).setFocus()
            source.reset()
            return False
        return True

