# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import utils


class Parties(utils.Tournoi):

    fichiersresultats = []

    def __init__(self):
        super(Parties, self).__init__()

    def preambuleInitierPartie(self, source):
        ''''Séquence d'avertissement avant initiation proprement dite'''
        partie = self.infos['derpartie']
        # récupérer les informations utiles depuis le formulaire d'appel
        form = source.Model.Parent
        form.reload()
        nbJoueurs = form.getInt(form.findColumn("Licences"))
        nbTables = form.getInt(form.findColumn("Tables"))
        # la partie doit être nommée
        if self.infos['nom'] == self.starttext:
            self.msgbox("La partie doit impérativement être nommée.\n"
                        "Le nom de la compétition doit être fourni dans\t\n"
                        "le module Paramètres (icone en haut à droite).", "Erreur", 'warn')
            return
        # des joueurs doivent avoir été  inscrits
        elif not nbJoueurs:
            self.msgbox("Aucun inscrit !", "Erreur",'warn')
            return
        # les tables doivent toutes avoir été attribuées
        elif partie == 0 and nbTables < nbJoueurs:
            self.msgbox ("Les tables n'ont pas été ou sont incomplètement attribuées.\t", "Erreur", 'warn')
            return
        # il doit rester des parties à jouer
        elif partie == self.infos['maxpartie']:
            self.msgbox ("Toutes les parties ont été initiées.")
            return
        # les résultats de la partie en cours devraient idéalement être clôturés
        elif partie > 0:
            tBord = form.Parent.getByName("tBord")
            tBord.moveToBookmark(partie)
            if tBord.getString( tBord.findColumn("Résultats") ) != "COMPLETS":
                if self.msgbox("\nVous allez initialiser la partie %d\n"
                               "alors que le classement de la partie %d\t\n"
                               "n'a pas été entièrement édité.\n\n"
                               "Souhaitez-vous poursuivre ?\n\n" % (partie+1,partie),
                               "Nouvelle partie", 'query', ('yesno', 'defno')) == 3: return
        # la routine d'initiation proprement dite
        self.initierPartie( partie+1 )
        # mise à jour de l'affichage
        form.Parent.getByName("tBord").reload()
        form.Parent.getByName("Standard_Menu_Principal").reload()

    def initierPartie(self, partie):
        '''Transfère les n° de tables adéquats suivant le type de tournoi et
        signale au programme que l'encodage peut commencer.
        Paramètre : numéro de la partie à initier'''
        from time import sleep
        bdp = self.barreDeProgression('Initialisation...\t', 3)
        bdp.start()
        bdp.setValue(1)
        if partie == 1:
            self.executeQuery( 'UPDATE "T_Parties" SET "T%d" = "TableEnCours"' % partie)
            self.executeQuery( 'UPDATE "T_Parties" SET "TableProvisoire" = "TableEnCours"')
        else:
            if self.infos['tablesaumerite'] and not self.infos['suspredistr']:
                self.executeQuery( 'UPDATE "T_Parties" SET "T%d" = "TableProvisoire"' % partie)
                self.executeQuery( 'UPDATE "T_Parties" SET "TableEnCours" = "TableProvisoire"' )
            else:   
                self.executeQuery( 'UPDATE "T_Parties" SET "T%d" = "TableEnCours"' % partie)
        bdp.setValue(2)
        # mettre à jour la table d'infos
        self.executeQuery( 'UPDATE "T_Infos" SET "PartiesJouees" = %d' % partie )
        self.executeQuery( 'UPDATE "T_Infos" SET "SuspRedistr" = False' )
        # pas certain que ce soit utile, commenté provisoirement
        # #actualiserTables
        bdp.setTextValue('Partie initiée\t', 3)
        sleep(0.5)
        bdp.end()

    def annulerDerPartie(self):
        '''Avertissements avant annulation de la dernière partie initiée'''
        from time import sleep
        partie = self.infos['derpartie']
        if not partie:
            self.msgbox("Aucune partie n'a encore été initiée.\t", "Erreur", "warn")
        else:
            if self.msgbox ("\nToutes les données de la partie %d\nvont être effacées et cette partie\t\n"
                            "devra être réinitiée.\n\nSouhaitez-vous poursuivre ?\n " % partie,
                            "Avertissement", "warn", boutons=('yesno','defno')) == 2:
                bdp = self.barreDeProgression('Annulation...\t', 3)
                bdp.start()
                # la routine d'annulation proprement dite
                bdp.setValue(1)
                self.effacePartie(partie)
                # si module Parties ouvert, recharge les données du tableau de bord
                bdp.setValue(2)
                self.forms_reload()
                bdp.setTextValue('Annulation réussie\t', 3)
                sleep(0.75)
                bdp.end()

    def effacePartie(self, i):
        '''Annuler la partie i par instructions SQL'''
        # supprimer les données
        self.executeQuery('UPDATE "T_Parties" SET "T{0}" = null, "R{0}" = null, "P{0}" = null'.format(i))
        # recalculer le cumul
        self.recalculCumul()
        # mettre à jour l'attribution des tables provisoire (quand changement entre parties)
        if i > 1:
            self.executeQuery('UPDATE "T_Parties" SET "TableEnCours" = "T%d"' % (i-1) )
            self.xTableProvisoire(i-1)
        # dans tous les cas, mettre à jour les données de "T_Infos"
        self.executeQuery('UPDATE "T_Infos" SET "PartiesJouees" = %d' % (i-1) )

    def calculeRang(self, partie=0):
        '''Calcule le classement de la partie passée en paramètre.
        On récupère les enregistrements, triés par points puis par n° de table, sans la ligne de top.
        Le classement est injecté dans la colonne Rx de la partie concernée.'''
        partie = partie or self.infos['derpartie']
        SQL = ('UPDATE "T_Parties" SET "R{0}" = SELECT "rang"\n'
               'FROM (SELECT "a"."ID_Partie" AS "ID", COUNT( "b"."P{0}" )+1 AS "rang"\n'
               '      FROM "T_Parties" AS "a"\n'
               '            LEFT JOIN "T_Parties" AS "b"\n'
               '                 ON "a"."P{0}" < "b"."P{0}" AND "b"."ID_Partie" > 0\n'
               '      GROUP BY "a"."ID_Partie") as "d"\n'
               'WHERE "ID" = "ID_Partie" AND "ID_Partie" > 0 AND "P{0}" IS NOT NULL\n').format(partie)
        self.executeQuery(SQL)
        # calcul du cumul
        self.recalculCumul()

    def recalculCumul(self):
        '''recalculer le cumul après mise à jour d'une partie'''
        SQL = 'UPDATE "T_Parties" set "Cumul" = '
        SQL += ' + '.join('coalesce("P%d",0)' % (p+1) for p in range(self.infos['maxpartie']))
        self.executeQuery(SQL)

    def xTableProvisoire(self, n):
        '''Recalcule l'ordre des tables (sauf tables fixes)
        si changement de tables entre parties'''
        # version simple si tables fixes indépendantes
        if self.infos['tablesparalleles']:
            # réattribue les tables non fixes uniquement
            rowset = self.executeQuery('SELECT "T%(partie)d" FROM "T_Parties" WHERE "Fixe" = FALSE '
                                       'ORDER BY "Cumul" DESC, "T%(partie)d" ASC' % {'partie':n})
            sql = ''
            update = 'UPDATE "T_Parties" SET "TableProvisoire" = %d WHERE "T%d" = %s AND "Fixe" = FALSE;\n'
            z = 1
            while rowset.next():
                table = rowset.getString(1)
                sql += update % (z, n, table)
                z += 1
            self.execute(sql)
        # tables fixes dans la numérotation générale
        else:
            # récupère la liste des tables fixes effectivement attribuées
            rowset = self.executeQuery('SELECT "T%d" FROM "T_Parties" WHERE "Fixe" = TRUE' % n)
            tablesfixes = []
            while rowset.next():
                tablesfixes.append(rowset.getShort(1))
            # Récupérer les tables non fixes sous la forme d’un tuple de tuples (table, cumul, Tn) :
            # >>> tablesmobiles = ((1,890, 3),(2,891, 2,),(3, 853,17),...) -> classement sur cumul puis sur Tn
            rowset = self.executeQuery('SELECT "TableEnCours", "Cumul", "T%d" FROM "T_Parties" WHERE "Fixe" = False' % n)
            tablesmobiles = []
            while rowset.next():
                tablesmobiles.append((rowset.getShort(1),rowset.getShort(2),rowset.getShort(3)))
            # Récupérer un mappage (ancienne table, nouvelle table)
            # [utiliser « range() » pour éviter de devoir gérer les tables vides]
            tablesattribuables = iter(set(range(1,len(tablesmobiles)+ len(tablesfixes)+1))-set(tablesfixes))
            tablifier = lambda x: (x[0], next(tablesattribuables))
            reattribution = map(tablifier, sorted(tablesmobiles, key=lambda x:(x[1],-x[2]), reverse=True))
            # Injecter les nouvelles tables
            SQL = ''
            for anctable, nouvtable in reattribution:
                SQL += 'UPDATE "T_Parties" SET "TableProvisoire" = %d WHERE "T%d" = %d;\n' % (nouvtable, n, anctable)
            self.executeQuery(SQL)
            # Mettre à jour les tables fixes également, dans le cas rare mais éventuel
            # où une table est "figée" en cours de compétition.
            SQL = 'UPDATE "T_Parties" SET "TableProvisoire" = "TableEnCours" WHERE "Fixe" = TRUE'
            self.executeQuery(SQL)

    def importerResultats(self, source):
        form = source.Model.Parent
        if not self.infos['derpartie']:
            self.msgbox("Aucune partie n'a encore été initiée.\t", "Erreur", 'warn')
            return
        tbord = form.Parent.getByName("tBord")
        tbord.moveToBookmark(self.infos['derpartie'])
        if tbord.getString(tbord.findColumn("Résultats")) == "COMPLETS":
            if self.msgbox("\nTous les résultats ont été encodés.\n"
                           "S'il s'agit des résultats d'une nouvelle partie, il vous faut d'abord l'initier.\t\n"
                           "\n\nContinuer malgré tout ?\n\n",
                           "Avertissement", 'query', boutons=('yesno','defno')) == 3:
                return
        rowset = self.executeQuery('SELECT "P%d" FROM "T_Parties" WHERE "ID_Partie" = 0' % self.infos['derpartie'])
        rowset.next()
        TOP = rowset.getInt(1)
        if not TOP:
            TOP = self.inputbox(texte="Veuillez saisir le top de la manche %d :" % self.infos['derpartie'])
            if TOP.isdigit():
                self.execute('UPDATE "T_Parties" SET "P%d" = %s WHERE "ID_Partie" = 0' % (self.infos['derpartie'], TOP))
            else:
                self.msgbox("Le top n'a pas été encodé.\n"
                            "Merci de le faire manuellement.", "Avertissement", 'warn')
        # ' la routine d'importation des données proprement dite
        self.lectureFichiers()

    def lectureFichiers(self):
        '''Récupère les points depuis un fichier txt DUPLITOP.
        Une table vide doit renvoyer 0.
        Les lignes vides seront ignorées.'''
        import uno
         
        def miseAJourFichiersLus(valide):
            '''Met à jour le listing des fichiers lus
            dans le dialogue de sélection de fichiers'''
            # 'le nom du fichier source
            fichier = os.path.basename(self.convertFromUrl(filename))
            fichiersLus = dlg.getControl("txtFichiers").getText()  # liste des fichiers déjà lus
            if valide:
                fichierLu = "%s-%s\t\t\t(%s)" % (data[0][0],data[-1][0],fichier)
            else:
                fichierLu = "[fichier invalide]\t\t\t(%s)" % fichier
            # 'on n'écrit pas 2x le même fichier
            if not fichierLu in fichiersLus:
                lignes = fichiersLus and fichiersLus.split('\n') or []
                lignes.append(fichierLu)
                lignes.sort()
                dlg.getControl("txtFichiers").setText('\n'.join(lignes))

        irow = 0
        # ' lance le dialogue d'importation A PLACER DANS LA ROUTINE QUI SUIT !
        dlg = self.createTooournoiDialog('dialogue3')
        liste = dlg.getControl('listeparties')
        liste.addItems( tuple(range(1,self.infos['derpartie']+1)), 0 )
        liste.selectItem(self.infos['derpartie'], True)
        liste.DropDownLineCount = liste.ItemCount #cosmétique
        dlg.getControl("btnFileControl").setFocus()
        # ' préparation de la requête UPDATE
        sql = ''
        if self.infos['tablesparalleles']:
            update = 'UPDATE "T_Parties" SET "P%s" = %s WHERE "T%s" = %s AND "Fixe" = %s;\n'
        else:
            update = 'UPDATE "T_Parties" SET "P%s" = %s WHERE "T%s" = %s;\n'
        while 1:
            while 1:
                if not dlg.execute():
                    # ' un fichier au moins a été lu ?
                    if irow:
                        self.calculsEtMisesAJour(liste.SelectedItem)
                    return
                elif not self.fichiersresultats:
                    self.msgbox ("Fichier inconnu.\t", "Erreur", 'warn')
                # '...sinon traitement
                else:
                    dlg.getControl('BoutonAnnuler').setLabel('~Fermer')
                    # récupérer le n° de la partie dont on importe les résultats
                    partie = liste.SelectedItem
                    break
            for n, filename in enumerate(self.fichiersresultats,1):
                # ' vérifier nature fichier avec SimpleFileAccess
                oTypeDetect = self.createUnoService("com.sun.star.document.TypeDetection")
                if "writer" in oTypeDetect.queryTypeByURL(filename):
                    if "tables" in filename.lower():
                        oDoc = self.importViaCalc(filename, Hidden=True)
                        f = oDoc.Sheets.getByIndex(0)
                        c = f.createCursor()
                        c.gotoStartOfUsedArea(False)
                        c.gotoEndOfUsedArea(True)
                        # 'récupère le données sous forme d'un array (trié car les tables peuvent être en désordre)
                        # transformation en listes de chaînes
                        # (un peu de chipo car pyuno convertit les doubles en floats)
                        # ajouté 'tuple(...)' pour compatibilité avec LibO (Python 3.3)
                        data = tuple(map(lambda x: [str(int(y)) if isinstance(y,float) else y for y in x], sorted(c.getDataArray())))
                        # ' parcours le dataArray et insère les points au fur et à mesure
                        bdp = self.barreDeProgression("Récupération des données...\t", len(data))
                        bdp.start()
                        for irow, (table, joueur, points) in enumerate(data, 1):
                            # ' on vérifie qu'un numéro de table valable est fourni
                            if table:
                                t = table.split(self.infos['symbolefixe'])[-1]
                                # ' Duplitop fournit -1 comme point d'une table non attribuée
                                if not points or int(points) < 1: points = '0'
                                # ' récupère l'enregistrement correspondant à la table
                                if self.infos['tablesparalleles']:  # distinction TRUE/FALSE, à proscrire dans les autres cas
                                    fixe = not table.isdigit()
                                    sql += update % (partie, points, partie, t, fixe)
                                else:
                                    sql += update % (partie, points, partie, t)
                                bdp.setValue(irow)
                        # ' on exécute la requête
                        self.execute(sql)
                        # ' routine de mise à jour des fichiers lus dans le dialogue de sélection de fichiers
                        miseAJourFichiersLus(True)
                        bdp.end()
                        if oDoc: oDoc.close(True)
                    else:
                        miseAJourFichiersLus(False)
                else:
                    miseAJourFichiersLus(False)
            dlg.getControl("btnFileControl").setFocus()

    def importerResultatsDuClasseur(self, source):
        '''
        BETA
        Permet de modifier les points à partir du classeur Partie
        '''
        import classements
        doc = self.doc
        partie  = self.getTournoiInfo('ClassementPartie') # = champ lié au sélecteur de partie
        # self.callXray(partie)
        # return
        # valider une éventuelle saisie en cours
        self.executeDispatch(".uno:Deselect")
        # récupérer les données
        # (un peu de chipo car pyuno convertit les doubles en floats)
        dataname = source.Model.Tag
        data = doc.DatabaseRanges.getByName(dataname).ReferredCells.getDataArray()
        data = map(lambda x: [str(int(y)) if isinstance(y,float) else y for y in (x[0],x[-1])], data[1:])
        ## traitement
        # top
        sql = 'UPDATE "T_Parties" SET "P%s" = %s WHERE "T%s" = 0;\n' % (partie, data[0][1], partie)
        # joueurs
        if self.infos['tablesparalleles']:
            update = 'UPDATE "T_Parties" SET "P%s" = %s WHERE "T%s" = %s AND "Fixe" = %s;\n'
        else:
            update = 'UPDATE "T_Parties" SET "P%s" = %s WHERE "T%s" = %s;\n'
        for table, points in data[1:]:
            if points.isdigit():
                t = table.split(self.infos['symbolefixe'])[-1]
                if self.infos['tablesparalleles']:  # distinction TRUE/FALSE, à proscrire dans les autres cas
                    fixe = not table.isdigit()
                    sql += update % (partie, points, partie, t, fixe)
                else:
                    sql += update % (partie, points, partie, t)
        # ' on exécute la requête
        self.execute(sql)
        self.calculeRang(partie)
        self.xTableProvisoire(self.infos['derpartie'])
        self.forms_reload()
        classements.rafraichirClassements(categ=True)
        self.doc.store()
        self.msgbox('\nPARTIE %s\n\n'
                    'Modifications enregistrées.\n' % partie)

    def calculsEtMisesAJour(self, partie):
        '''Injecte les scores dans la table T_Partie'''
        bdp = self.barreDeProgression("Actualisation des données...\t", 3)
        bdp.start()
        bdp.setValue(1)
        self.calculeRang(partie)
        bdp.setValue(2)
        self.xTableProvisoire(self.infos['derpartie'])
        bdp.setTextValue("Sauvegarde...\t", 3)
        utils.databasedocument.store()
        # ' si module Parties ouvert, recharge les données du tableau de bord
        self.doc.DrawPage.Forms.getByName("Total").reload()
        self.doc.DrawPage.Forms.getByName("tBord").reload()
        bdp.end()

    def choixPartieSaisiePoints(self, source):
        '''Sélection de la partie pour l'encodage des points'''
        # ' vérifie source de l'événement : document (chargement) ou bouton
        if source.supportsService("com.sun.star.form.FormComponent"):
            form = source
            partie = self.infos['derpartie']
        else:
            form = source.Model.Parent
            while 1:
                partie = self.inputbox("Partie n° :", self.infos['derpartie'])
                # ' si valeur non valide...
                if not partie:
                    return
                elif not partie.isdigit() or int(partie) < 1:
                    return
                elif int(partie) > self.infos['maxpartie']:
                    self.msgbox("Ce tournoi se joue en %s.\t\n" % 
                                ("1 partie" if self.infos['maxpartie'] == 1 else "%d parties" % self.infos['maxpartie']),
                                "Erreur", 'warn')
                elif int(partie) > self.infos['derpartie']:
                    self.msgbox("Cette partie n'a pas encore été initiée !\t\n", "Erreur", 'warn')
                else: break
        # ' prépare la requête en fonction de la partie souhaitée
        self.infos['partie'] = partie
        tri = '"T%s" ASC' % partie
        if self.infos['tablesparalleles']:
            tri = '"Fixe" ASC, ' + tri
        if self.infos['tablesaumerite'] or self.infos['tablesparalleles']:
            SQL = ('SELECT "ID_Partie", CASEWHEN("ID_Partie" = 0, \'TOP\',\n'
                   'CASEWHEN("Fixe",\'%(symbolefixe)s\' || "T%(partie)s", \'\' || "T%(partie)s")) AS "Tables",\n'
                   '"P%(partie)s" AS "Points", "Licence" FROM "T_Parties"')
        else:
            SQL = ('SELECT "ID_Partie", CASEWHEN("ID_Partie" = 0, \'TOP\', \'\' || "T%(partie)s") AS "Tables",\n'
                   '"P%(partie)s" AS "Points", "Licence" FROM "T_Parties"')
        if int(partie) > 1:
            SQL += '\nWHERE "Cumul" > 0 OR "P1" IS NULL'
        form.Command = SQL % self.infos     # modifie la requête source
        form.Order = tri
        form.getByName('lblTitre').Label = "Partie %s" % partie     # modifie le titre
        form.getByName('Grid_Saisie_Points').getByName('colJoueur').Hidden = self.infos['paires']
        form.getByName('Grid_Saisie_Points').getByName('colPaires').Hidden = not self.infos['paires']
        form.reload()   # rafraîchit l'affichage

    def recalculsSaisiePoints(self, source):
        '''Recalculs après saises'''
        # ' ouvre barre de progression
        bdp = self.barreDeProgression("Recalcul du classement...", 10)
        bdp.start()
        # ' récupère les n° de parties
        form = source.Model.Parent
        n = int(form.getByName('lblTitre').Label[7:])
        # ' enregistre si nécessaire la dernière saisie
        self.doc.CurrentController.getControl(form.getByName("Grid_Saisie_Points")).commit()
        if form.IsNew: form.insertRow()
        else: form.updateRow()
        bdp.setValue(4)
        # ' calcule le rang de la partie concernée
        self.calculeRang(n)
        bdp.setValue(7)
        # ' recalcule l'attribution des tables 
        # ' toujours le faire à partir de la dernière partie initiée
        # ' (important si on modifie une partie antérieure)
        self.xTableProvisoire(self.infos['derpartie'])
        # ' si module Parties ouvert, recharge les données du tableau de bord
        self.forms_reload()
        bdp.setValue(8)
        # ' force la sauvegarde
        bdp.setTextValue("Sauvegarde de la base...", 9)
        utils.databasedocument.store()
        self.executeDispatch(".uno:CloseDoc", frame=form.Parent.Parent.CurrentController.Frame)


